@@ -21,8 +21,8 @@
 
 
 /*
- *  There are three types of web hooks: web hook functions, web accessable
- *  variables, and post file handlers.
+ *  There are four types of web hooks: web hook functions, web accessable
+ *  variables, post file handlers, and directory listing handlers.
  */
 
 /*
@@ -71,6 +71,19 @@ extern hook_type lookup_web_hook(const char *hook_name);
 extern void init_cgi(char *query, size_t init_count);
 extern char * get_cgi(char *name);
 extern void set_cgi(char *name, char *value);
+extern int count_cgi(void);
+
+/*
+ *  The following four functions allow the web server to communicate to hooks
+ *  the username and password, if any, that were used to authenticate with the
+ *  web server.  They can be NULL if no authentication was provided.  The first
+ *  two are for use by the hooks, to read the username and password, and the
+ *  second two are for use by the web server, to set the username and password.
+ */
+extern const char *current_web_username(void);
+extern const char *current_web_password(void);
+extern void set_current_web_username(char *new_username);
+extern void set_current_web_password(char *new_password);
 
 /*
  *  The following four macros provide the four ways a hook can communicate with
@@ -181,13 +194,14 @@ enum
 	RESTART_NAS = 0x40,
 	RESTART_SYSLOG = 0x80,
 	RESTART_NETWORKING = 0x100,
-	RESTART_UPNP_AV = 0x200
+	RESTART_UPNP_AV = 0x200,
+	RESTART_FTP = 0x400
 };
 
 #define RESTART_ALL \
 		(RESTART_SAMBA | RESTART_NFS | RESTART_HTTPD | RESTART_DNS | \
 		 RESTART_DHCPD | RESTART_UPNP | RESTART_NAS | RESTART_SYSLOG | \
-		 RESTART_NETWORKING | RESTART_UPNP_AV)
+		 RESTART_NETWORKING | RESTART_UPNP_AV | RESTART_FTP)
 
 /*
  *  The ARGV() macro is a helper utility for packaging up arguments for the
@@ -352,8 +366,10 @@ extern int websJavascriptCall(webs_t wp, char *function, char *format, ...);
  *  The post_file_handler_type type defines the type that each post file
  *  handler function must have.  When it is activated, it is passed the URL in
  *  the url parameter, a pointer to the stream from which the data for the file
- *  may be read in the ``stream'' parameter, and a number of bytes in the file
- *  in the ``len'' parameter.
+ *  may be read in the ``stream'' parameter, a number of bytes in the file in
+ *  the ``len'' parameter, the name of the file being posted in the
+ *  ``file_name'' parameter, and the boundary string in the ``boundary''
+ *  parameter.
  *
  *  The register_post_file_handler() function is called to register a post file
  *  handler.  All such calls should be made during initialization time, not
@@ -362,7 +378,8 @@ extern int websJavascriptCall(webs_t wp, char *function, char *format, ...);
  *  The lookup_post_file_handler() is for use by the web server, to look up the
  *  handler for a particular file post operation.
  */
-typedef void (*post_file_handler_type)(char *url, FILE *stream, int len);
+typedef void (*post_file_handler_type)(char *url, FILE *stream, int len,
+		const char *file_name, const char *boundary);
 
 extern void register_post_file_handler(const char *handler_name,
 		post_file_handler_type handler_function);
@@ -370,11 +387,103 @@ extern post_file_handler_type lookup_post_file_handler(
 		const char *handler_name);
 
 
+/*
+ *      Directory Listing Handlers
+ *      --------------------------
+ *
+ *  When the web server is providing a page for a share directory, it uses
+ *  three hooks: a header hook which is called to start the directory HTML
+ *  page; an element hook, which is called one time for each element (file or
+ *  directory) in the directory being listed; and a footer hook which is called
+ *  to complete the HTML page for the directory.
+ *
+ *  The types of these three hooks are specified by the
+ *  directory_header_handler_type, directory_element_handler_type, and
+ *  directory_footer_handler_type type declarations, respectively.  Each type
+ *  of hook is a function that has a void return type, and each takes as its
+ *  first argument a ``FILE *'' named ``stream''.  This is the stream to which
+ *  the output for the page should be sent.  Anything written to ``stream''
+ *  will be sent to the web client as part of the HTML page for the directory.
+ *  The header handler takes a second argument directory_url that specifies the
+ *  URL to the directory being listed.  This URL omits the ``http:'' and the
+ *  server name, but includes everything after that.  It will normally begin
+ *  with "/shares".  The element handler takes a second argument that is the
+ *  name of the element (i.e. the name of that file or directory within the
+ *  parent directory); a third argument that is the URL (again, starting with
+ *  "/shares") that can be used to access the element; and a fourth argument
+ *  that internal_path_to_element that specifies the path used to access the
+ *  element internally.  Note that internal_path_to_element should not be sent
+ *  out over the web -- instead, it can be used to stat() the element to find
+ *  out information about it, and that information can be sent out.  Finally,
+ *  the footer handler takes a second argument directory_url that is the same
+ *  as the directory_url argument to the header handler and a third argument
+ *  create_access that specifies whether or not create access is given to the
+ *  directory in question through the web interface.
+ *
+ *  For each of the header, element, and footer hooks, there is a
+ *  register_directory_*_handler() function which sets the handler to be used
+ *  and a lookup_directory_*_handler() function which returns the currently
+ *  registered handler.  If multiple handlers of a give type are registered,
+ *  the last one registered is the one used.
+ */
+typedef void (*directory_header_handler_type)(FILE *stream,
+											  const char *directory_url);
+typedef void (*directory_element_handler_type)(FILE *stream,
+		const char *element_name, const char *element_url,
+		const char *directory_url, const char *internal_path_to_element,
+		bool delete_access);
+typedef void (*directory_footer_handler_type)(FILE *stream,
+		const char *directory_url, bool create_access);
+
+extern void register_directory_header_handler(
+		directory_header_handler_type handler_function);
+extern directory_header_handler_type lookup_directory_header_handler(void);
+extern void register_directory_element_handler(
+		directory_element_handler_type handler_function);
+extern directory_element_handler_type lookup_directory_element_handler(void);
+extern void register_directory_footer_handler(
+		directory_footer_handler_type handler_function);
+extern directory_footer_handler_type lookup_directory_footer_handler(void);
+
+
+/*
+ *      Page Authorization Handlers
+ *      ---------------------------
+ *
+ *  A page authorization handler is a function that is called for certain pages
+ *  after the input for those pages has been read but before the page is
+ *  loaded.  The idea is that the function gets a chance to look at the input,
+ *  plus the username and password, and decide whether or not the user is
+ *  authorized to access that page.  If not, the authorization handler should
+ *  set the web_input_unauthorized global variable (see below) to TRUE.
+ *
+ *  Page authorization handlers are registered by name.  Image variables are
+ *  used to specify the mapping from URLs to handler names.  Here's an example.
+ *  These image variable settings:
+ *
+ *      web_page_authorizer_0_url=/mypage1.asp
+ *      web_page_authorizer_0_name=myhook1
+ *      web_page_authorizer_1_url=/mypage2.asp
+ *      web_page_authorizer_1_name=myhook1
+ *      web_page_authorizer_2_url=/mypage3.asp
+ *      web_page_authorizer_2_name=myhook2
+ *
+ *  This specifies that two pages, /mypage1.asp and /mypage2.asp, use the hook
+ *  registered under the name myhook1 and a third page, /mypage3.asp, uses the
+ *  hook registered under the name myhook2.
+ */
+typedef void (*page_authorizer_type)(webs_t wp);
+
+extern void register_page_authorizer(const char *name,
+									 page_authorizer_type function);
+extern page_authorizer_type lookup_page_authorizer(const char *name);
+
+
 /*
  *      General Hook Support
  *      --------------------
  *
- *  The following six variables can be used by hooks to communicate with the
+ *  The following seven variables can be used by hooks to communicate with the
  *  core server code.  They can be used by hook functions, validation functions
  *  for web accessable variables, and post file handlers.
  *
@@ -409,6 +518,14 @@ extern post_file_handler_type lookup_post_file_handler(
  *  sometimes required as a result of executing a hook but which shouldn't be
  *  done in the middle of serving the page because it may cause the web server
  *  to go down before it has completed serving the page.
+ *
+ *  The web_input_unauthorized flag can be set by a post file handler to
+ *  specify to the server that an unauthorized action is being attempted so
+ *  that the server can send the appropriate error header instead of a valid
+ *  result header.  Note that setting this anywhere but in a post file handler
+ *  will have no effect, because other hooks are executed after the headers
+ *  have been sent, when it's too late to indicate an authorization failure in
+ *  the header.
  */
 extern u32 restart_needed_bits;
 extern bool reboot_needed;
@@ -416,6 +533,7 @@ extern bool complete_restart_needed;
 extern bool commit_needed;
 extern bool non_nvram_changes_made;
 extern void (*end_of_page_hook)(void);
+extern bool web_input_unauthorized;
 
 
 #endif /* WEBHOOKS_H */
