@@ -51,3 +51,5 @@ mkdir -p proc
 mkdir -p shares
 mkdir -p foreign_shares
 
+# for vsftpd - BBB
+mkdir -p usr/share/empty
