@@ -28,6 +28,23 @@
 #include <bcmnvram.h>
 #include <nvparse.h>
 
+#include "webhooks/webhooks.h"
+
+/*
+ * Concatenate two strings together into a caller supplied buffer
+ * @param	s1	first string
+ * @param	s2	second string
+ * @param	buf	buffer large enough to hold both strings
+ * @return	buf
+ */
+static inline char * strcat_r(const char *s1, const char *s2, char *buf)
+{
+	strcpy(buf, s1);
+	strcat(buf, s2);
+	return buf;
+}	
+
+
 /* Copy each token in wordlist delimited by space into word */
 #define foreach(word, wordlist, next) \
 	for (next = &wordlist[strspn(wordlist, " ")], \
@@ -214,6 +231,8 @@ set_autofw_port(int which, const netconf_app_t *app)
 	char name[] = "autofw_portXXXXXXXXXX", value[1000], *cur = value;
 	int len;
 
+	var_hook_info autofw_port_info;
+
 	if (!valid_autofw_port(app))
 		return FALSE;
 
@@ -265,9 +284,12 @@ set_autofw_port(int which, const netconf_app_t *app)
 		cur = safe_snprintf(cur, &len, app->desc);
 	}
 
-	/* Do it */
-	if (nvram_set(name, value))
-		return FALSE;
+	autofw_port_info.name = name;
+	autofw_port_info.restart_needed_bits =  RESTART_NETWORKING;
+	autofw_port_info.nullok = TRUE;
+    
+	hook_set_var(&autofw_port_info,value);
+
 
 	return TRUE;
 }
@@ -408,6 +430,8 @@ set_forward_port(int which, const netconf_nat_t *nat)
 	char name[] = "forward_portXXXXXXXXXX", value[1000], *cur = value;
 	int len;
 
+	var_hook_info forward_info;
+
 	if (!valid_forward_port(nat))
 		return FALSE;
 
@@ -450,9 +474,11 @@ set_forward_port(int which, const netconf_nat_t *nat)
 		cur = safe_snprintf(cur, &len, nat->desc);
 	}
 
-	/* Do it */
-	if (nvram_set(name, value))
-		return FALSE;
+	forward_info.name = name;
+	forward_info.restart_needed_bits = RESTART_NETWORKING;
+	forward_info.nullok = TRUE;
+
+	hook_set_var(&forward_info,value);
 
 	return TRUE;
 }
@@ -686,6 +712,7 @@ set_filter_client(int which, const netconf_filter_t *start, const netconf_filter
 {
 	char name[] = "filter_clientXXXXXXXXXX", value[1000], *cur = value;
 	int len;
+	var_hook_info filter_info;
 
 	if (!valid_filter_client(start, end))
 		return FALSE;
@@ -742,9 +769,10 @@ set_filter_client(int which, const netconf_filter_t *start, const netconf_filter
 		cur = safe_snprintf(cur, &len, start->desc);
 	}
 
-	/* Do it */
-	if (nvram_set(name, value))
-		return FALSE;
+	filter_info.name = name;
+	filter_info.restart_needed_bits = RESTART_NETWORKING;
+	filter_info.nullok = TRUE;
+	hook_set_var(&filter_info,value);
 
 	return TRUE;
 }
@@ -758,6 +786,106 @@ del_filter_client(int which)
 	return (nvram_unset(name) == 0) ? TRUE : FALSE;
 }
 
+/*
+ * wl_wds<N> is authentication protocol dependant.
+ * when auth is "psk":
+ *	wl_wds<N>=mac,role,crypto,auth,ssid,passphrase
+ */
+bool
+get_wds_wsec(int unit, int which, char *mac, char *role, 
+	     char *crypto, char *auth, ...)
+{
+	char name[] = "wlXXXXXXX_wdsXXXXXXX", value[1000], *next;
+	
+	snprintf(name, sizeof(name), "wl%d_wds%d", unit, which);
+	strncpy(value, nvram_safe_get(name), sizeof(value));
+	next = value;
+
+	/* separate mac */
+	strcpy(mac, strsep(&next, ","));
+	if (!next)
+		return FALSE;
+	
+	/* separate role */
+	strcpy(role, strsep(&next, ","));
+	if (!next)
+		return FALSE;
+
+	/* separate crypto */
+	strcpy(crypto, strsep(&next, ","));
+	if (!next)
+		return FALSE;
+
+	/* separate auth */
+	strcpy(auth, strsep(&next, ","));
+	if (!next)
+		return FALSE;
+
+	if (!strcmp(auth, "psk")) {
+		va_list va;
+
+		va_start(va, auth);
+
+		/* separate ssid */
+		strcpy(va_arg(va, char *), strsep(&next, ","));
+		if (!next)
+			goto fail;
+
+		/* separate passphrase */
+		strcpy(va_arg(va, char *), next);
+
+		va_end(va);
+		return TRUE;
+fail:
+		va_end(va);
+		return FALSE;
+	}
+
+	return FALSE;
+}
+
+bool
+set_wds_wsec(int unit, int which, char *mac, char *role,
+	     char *crypto, char *auth, ...)
+{
+	char name[] = "wlXXXXXXX_wdsXXXXXXX", value[10000];
+
+	snprintf(name, sizeof(name), "wl%d_wds%d", unit, which);
+	snprintf(value, sizeof(value), "%s,%s,%s,%s",
+		 mac, role, crypto, auth);
+
+	if (!strcmp(auth, "psk")) {
+		int offset;
+		va_list va;
+
+		va_start(va, auth);
+		offset = strlen(value);
+		snprintf(&value[offset], sizeof(value) - offset, ",%s,%s",
+			 va_arg(va, char *), va_arg(va, char *));
+		va_end(va);
+
+		if (nvram_set(name, value))
+			return FALSE;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+bool
+del_wds_wsec(int unit, int which)
+{
+	char name[] = "wlXXXXXXX_wdsXXXXXXX";
+
+	snprintf(name, sizeof(name), "wl%d_wds%d", unit, which);
+
+	nvram_unset(name);
+
+	return TRUE;
+}
+
+
 static void
 convert_filter_ip(void)
 {
@@ -1096,6 +1224,93 @@ convert_autofw_port(void)
 	}
 }
 
+static void
+convert_pppoe(void)
+{
+	char *old[] = {
+		"pppoe_ifname",		/* PPPoE enslaved interface */
+		"pppoe_username",	/* PPP username */
+		"pppoe_passwd", 	/* PPP password */
+		"pppoe_idletime",	/* Dial on demand max idle time (seconds) */
+		"pppoe_keepalive",	/* Restore link automatically */
+		"pppoe_demand",		/* Dial on demand */
+		"pppoe_mru",		/* Negotiate MRU to this value */
+		"pppoe_mtu",		/* Negotiate MTU to the smaller of this value or the peer MRU */
+		"pppoe_service",	/* PPPoE service name */
+		"pppoe_ac",		/* PPPoE access concentrator name */
+		NULL
+	};
+	char new[] = "wan_pppoe_XXXXXXXXXXXXXXXXXX";
+	char *value;
+	int i;
+	for (i = 0; old[i] != NULL; i ++) {
+		if ((value = nvram_get(old[i]))) {
+			snprintf(new, sizeof(new), "wan_%s", old[i]);
+			nvram_set(new, value);
+			nvram_unset(old[i]);
+		}
+	}
+}
+
+/*
+ * convert wl_wep - separate WEP encryption from WPA cryptos:
+ * 	wep|on|restricted|off -> enabled|disabled
+ * 	tkip|ase|tkip+aes -> wl_crypto
+ * combine wl_auth_mode and wl_auth:
+ *	wl_auth 0|1 -> wl_auth_mode open|shared (when wl_auth_mode is disabled)
+ */
+static void
+convert_wsec(void)
+{
+	char prefix[] = "wlXXXXXXXXXX_", tmp[64], *wep;
+	int i;
+
+	for (i = 0; i < MAX_NVPARSE; i ++) {
+		sprintf(prefix, "wl%d_", i);
+		wep = nvram_get(strcat_r(prefix, "wep", tmp));
+		if (!wep)
+			continue;
+		/* 3.60.xx */
+		/* convert wep, restricted, or on to enabled */
+		if (!strcmp(wep, "wep") || !strcmp(wep, "restricted") ||
+		    !strcmp(wep, "on"))
+			nvram_set(strcat_r(prefix, "wep", tmp), "enabled");
+		/* split wep and wpa to wl_wep and wl_crypto */
+		else if (!strcmp(wep, "tkip") || !strcmp(wep, "aes") ||
+			 !strcmp(wep, "tkip+aes")) {
+			nvram_set(strcat_r(prefix, "crypto", tmp), wep);
+			nvram_set(strcat_r(prefix, "wep", tmp), "disabled");
+		}
+		/* treat everything else as disabled */
+		else if (strcmp(wep, "enabled"))
+			nvram_set(strcat_r(prefix, "wep", tmp), "disabled");
+		/* combine 802.11 open/shared authentication mode with WPA to wl_auth_mode */
+		if (nvram_match(strcat_r(prefix, "auth_mode", tmp), "disabled")) {
+			if (nvram_match(strcat_r(prefix, "auth", tmp), "1"))
+				nvram_set(strcat_r(prefix, "auth_mode", tmp), "shared");
+			else
+				nvram_set(strcat_r(prefix, "auth_mode", tmp), "open");
+		}
+		/* 3.80.xx 
+		 *
+		 * check only if the wl_auth_mode is set to "shared"
+		 * wl_auth_carries a default value of "0" 
+		 */
+		/* split 802.11 auth from wl_auth_mode */
+		if (nvram_match(strcat_r(prefix, "auth_mode", tmp), "shared"))
+			nvram_set(strcat_r(prefix, "auth", tmp), "1");
+
+		/* split wpa akm from wl_auth_mode */
+		if (nvram_match(strcat_r(prefix, "auth_mode", tmp), "wpa"))
+			nvram_set(strcat_r(prefix, "akm", tmp), "wpa");
+		else if (nvram_match(strcat_r(prefix, "auth_mode", tmp), "psk"))
+			nvram_set(strcat_r(prefix, "akm", tmp), "psk");
+		/* preserve radius only in wl_auth_mode */
+		if (nvram_invmatch(strcat_r(prefix, "auth_mode", tmp), "radius"))
+			nvram_set(strcat_r(prefix, "auth_mode", tmp), "none");
+	}
+}
+
 void
 convert_deprecated(void)
 {
@@ -1123,4 +1338,10 @@ convert_deprecated(void)
 
 	/* autofw_port were single destination port  and now support a port range */
 	convert_autofw_port();
+
+	/* pppoe_XXXXXXXX are now wan_pppoe_XXXXXXXX */
+	convert_pppoe();
+
+	/* convert wl_wep and combine wl_auth_mode and wl_auth */
+	convert_wsec();
 }
