@@ -1,5 +1,5 @@
 /*
- * Copyright 2004, Broadcom Corporation
+ * Copyright 2005, Broadcom Corporation
  * All Rights Reserved.
  *
  * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
@@ -34,28 +34,36 @@
 /* See documentation for Device Type values, 32 values max */
 #ifndef	HPNA_DEV_TYPE
 
+#if	defined(CONFIG_BRCM_VJ)
+#define HPNA_DEV_TYPE		{ CDCF_V0_DEVICE_DISPLAY }
+
+#elif	defined(CONFIG_BCRM_93725)
+#define HPNA_DEV_TYPE		{ CDCF_V0_DEVICE_CM_BRIDGE, CDCF_V0_DEVICE_DISPLAY }
+
+#else
 #define HPNA_DEV_TYPE		{ CDCF_V0_DEVICE_PCINIC }
 
+#endif
 
 #endif	/* !HPNA_DEV_TYPE */
 
 
 #define	EPI_MAJOR_VERSION	3
 
-#define	EPI_MINOR_VERSION	60
+#define	EPI_MINOR_VERSION	90
 
-#define	EPI_RC_NUMBER		13
+#define	EPI_RC_NUMBER		39
 
 #define	EPI_INCREMENTAL_NUMBER	0
 
 #define	EPI_BUILD_NUMBER	0
 
-#define	EPI_VERSION		3,60,13,0
+#define	EPI_VERSION		3,90,39,0
 
-#define	EPI_VERSION_NUM		0x033c0d00
+#define	EPI_VERSION_NUM		0x035a2700
 
 /* Driver Version String, ASCII, 32 chars max */
-#define	EPI_VERSION_STR		"3.60.13.0"
-#define	EPI_ROUTER_VERSION_STR	"3.61.13.0"
+#define	EPI_VERSION_STR		"3.90.39.0"
+#define	EPI_ROUTER_VERSION_STR	"3.91.39.0"
 
 #endif /* _epivers_h_ */
