@@ -22,20 +22,17 @@
 #include <proto/ethernet.h>
 #endif
 
-/* enable structure packing */
-#if !defined(__GNUC__)
-#pragma pack(1)
-#endif
+#include <proto/wpa.h>
 
-/* some platforms require stronger medicine */
+
+/* enable structure packing */
 #if defined(__GNUC__)
 #define	PACKED	__attribute__((packed))
 #else
+#pragma pack(1)
 #define	PACKED
 #endif
 
-#include <proto/wpa.h>
-
 #define DOT11_TU_TO_US			1024	/* 802.11 Time Unit is 1024 microseconds */
 
 /* Generic 802.11 frame constants */
@@ -388,6 +385,34 @@ typedef struct wme_params wme_params_t;
 
 #define WME_TXOP_UNITS	32
 
+/* AP: default params to be announced in the Beacon Frames/Probe Responses Table 12 WME Draft*/
+/* AP:  default params to be Used in the AP Side Table 14 WME Draft January 2004 802.11-03-504r5 */
+#define WME_AC_BK_ACI_STA       0x27
+#define WME_AC_BK_ECW_STA       0xA4
+#define WME_AC_BK_TXOP_STA      0x0000
+#define WME_AC_BE_ACI_STA       0x03
+#define WME_AC_BE_ECW_STA       0xA4
+#define WME_AC_BE_TXOP_STA      0x0000
+#define WME_AC_VI_ACI_STA       0x42
+#define WME_AC_VI_ECW_STA       0x43
+#define WME_AC_VI_TXOP_STA      0x005e
+#define WME_AC_VO_ACI_STA       0x62
+#define WME_AC_VO_ECW_STA       0x32
+#define WME_AC_VO_TXOP_STA      0x002f
+                                                                                                             
+#define WME_AC_BK_ACI_AP        0x27
+#define WME_AC_BK_ECW_AP        0xA4
+#define WME_AC_BK_TXOP_AP       0x0000
+#define WME_AC_BE_ACI_AP        0x03
+#define WME_AC_BE_ECW_AP        0x64
+#define WME_AC_BE_TXOP_AP       0x0000
+#define WME_AC_VI_ACI_AP        0x41
+#define WME_AC_VI_ECW_AP        0x43
+#define WME_AC_VI_TXOP_AP       0x005e
+#define WME_AC_VO_ACI_AP        0x61
+#define WME_AC_VO_ECW_AP        0x32
+#define WME_AC_VO_TXOP_AP       0x002f
+
 /* WME Traffic Specification (TSPEC) element */
 #define WME_SUBTYPE_TSPEC 2
 #define WME_TSPEC_HDR_LEN		2
@@ -702,6 +727,7 @@ struct dot11_management_notification {
 #define DOT11_MNG_IBSS_DFS_ID			41    /* 11H IBSS_DFS 		*/
 #define DOT11_MNG_ERP_ID			42
 #define DOT11_MNG_NONERP_ID			47
+#define DOT11_MNG_RSN_ID			48
 #define DOT11_MNG_EXT_RATES_ID			50
 #define DOT11_MNG_WPA_ID			221
 #define DOT11_MNG_PROPR_ID			221
@@ -792,7 +818,7 @@ typedef struct d11cnt {
 
 /* BRCM info element */
 struct brcm_ie {
-	uchar	id;		
+	uchar	id;		/* 221, DOT11_MNG_PROPR_ID */
 	uchar	len;   
 	uchar 	oui[3];
 	uchar	ver;
@@ -801,27 +827,45 @@ struct brcm_ie {
 } PACKED;
 #define BRCM_IE_LEN		8
 typedef	struct brcm_ie brcm_ie_t;
-#define BRCM_IE_VER		1
+#define BRCM_IE_VER		2
+#define BRCM_IE_LEGACY_AES_VER	1
 
 /* brcm_ie flags */
 #define	BRF_ABCAP		0x1	/* afterburner capable */
 #define	BRF_ABRQRD		0x2	/* afterburner requested */
+#define	BRF_LZWDS		0x4	/* lazy wds enabled */
+
+
+/* OUI for BRCM proprietary IE */
+#define BRCM_PROP_OUI		"\x00\x90\x4C"
+
+/* Vendor IE structure */
+struct vndr_ie {
+	uchar id;
+	uchar len;
+	uchar oui [3];
+	uchar data [1]; 	/* Variable size data */
+}PACKED;
+typedef struct vndr_ie vndr_ie_t;
+
+#define VNDR_IE_HDR_LEN		2	/* id + len field */
+#define VNDR_IE_MIN_LEN		3	/* size of the oui field */
+#define VNDR_IE_MAX_LEN		256
 
 /* WPA definitions */
 #define WPA_VERSION		1
 #define WPA_OUI			"\x00\x50\xF2"
 
-#define WPA_OUI_LEN	3
+#define WPA2_VERSION		1
+#define WPA2_VERSION_LEN	2
+#define WPA2_OUI		"\x00\x0F\xAC"
 
-/* WPA authentication modes */
-#define WPA_AUTH_NONE		0	/* None */
-#define WPA_AUTH_UNSPECIFIED	1	/* Unspecified authentication over 802.1X: default for WPA */
-#define WPA_AUTH_PSK		2	/* Pre-shared Key over 802.1X */
-#define WPA_AUTH_DISABLED	255	/* Legacy (i.e., non-WPA) */
+#define WPA_OUI_LEN	3
 				 
-#define IS_WPA_AUTH(auth)	((auth) == WPA_AUTH_NONE || \
-				 (auth) == WPA_AUTH_UNSPECIFIED || \
-				 (auth) == WPA_AUTH_PSK)
+/* RSN authenticated key managment suite */
+#define RSN_AKM_NONE		0	/* None (IBSS) */
+#define RSN_AKM_UNSPECIFIED	1	/* Over 802.1x */
+#define RSN_AKM_PSK		2	/* Pre-shared Key */
 
 
 /* Key related defines */
