@@ -4,7 +4,7 @@
  *
  * Definitions subject to change without notice.
  *
- * Copyright 2004, Broadcom Corporation
+ * Copyright 2005, Broadcom Corporation
  * All Rights Reserved.
  * 
  * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
@@ -22,10 +22,9 @@
 #include <proto/ethernet.h>
 #include <proto/802.11.h>
 
-#if defined(__GNUC__)
-#define	PACKED	__attribute__((packed))
-#else
-#define	PACKED
+/* require default structure packing */
+#if !defined(__GNUC__)
+#pragma pack(push,8)
 #endif
 
 #define WL_NUMRATES		255	/* max # of rates in a rateset */
@@ -45,39 +44,8 @@ typedef struct wl_rateset {
  * Per-bss information structure.
  */
 
-#define	WL_LEGACY_BSS_INFO_VERSION	106	/* an older supported version of wl_bss_info struct */
 #define	WL_BSS_INFO_VERSION		107	/* current version of wl_bss_info struct */
 
-typedef struct wl_bss_info106 {
-	uint		version;	/* version field */
-	struct ether_addr BSSID;
-	uint8		SSID_len;
-	uint8		SSID[32];
-	uint8		Privacy;	/* 0=No WEP, 1=Use WEP */
-	int16		RSSI;		/* receive signal strength (in dBm) */
-	uint16		beacon_period;	/* units are Kusec */
-	uint16		atim_window;	/* units are Kusec */
-	uint8		channel;	/* Channel no. */
-	int8		infra;		/* 0=IBSS, 1=infrastructure, 2=unknown */
-	struct {
-		uint	count;		/* # rates in this set */
-		uint8	rates[12];	/* rates in 500kbps units w/hi bit set if basic */
-	} rateset;			/* supported rates */
-        uint8           dtim_period;    /* DTIM period */
-	int8		phy_noise;	/* noise right after tx (in dBm) */
-	uint16		capability;	/* Capability information */
-	struct dot11_bcn_prb *prb;	/* probe response frame (ioctl na) */
-	uint16		prb_len;	/* probe response frame length (ioctl na) */
-	struct {
-		uint8 supported;	/* wpa supported */
-		uint8 multicast;	/* multicast cipher */
-		uint8 ucount;		/* count of unicast ciphers */
-		uint8 unicast[4];	/* unicast ciphers */
-		uint8 acount;		/* count of auth modes */
-		uint8 auth[4];		/* Authentication modes */
-	} wpa;
-} wl_bss_info106_t;
-
 typedef struct wl_bss_info {
 	uint32		version;	/* version field */
 	uint32		length;		/* byte length of data in this record, starting at version and including IEs */
@@ -208,7 +176,7 @@ typedef struct wl_rm_rep {
 #define WL_RM_REP_FIXED_LEN	8
 
 
-#if defined(WPAPSK)
+#if defined(BCMSUP_PSK)
 typedef enum sup_auth_status {
 	WLC_SUP_DISCONNECTED = 0,
 	WLC_SUP_CONNECTING,
@@ -218,7 +186,7 @@ typedef enum sup_auth_status {
 	WLC_SUP_KEYXCHANGE,
 	WLC_SUP_KEYED
 } sup_auth_status_t;
-#endif	/* CCX | WPAPSK */
+#endif	/* BCMCCX | BCMSUP_PSK */
 
 /* Enumerate crypto algorithms */
 #define	CRYPTO_ALGO_OFF			0
@@ -277,13 +245,16 @@ typedef struct {
 #define TKIP_ENABLED		2
 #define AES_ENABLED		4
 #define WSEC_SWFLAG		8
+#define SES_OW_ENABLED		0x0040	/* to go into transition mode without setting wep */
 
-#define WSEC_SW(wsec)		((wsec) & WSEC_SWFLAG)
-#define WSEC_HW(wsec)		(!WSEC_SW(wsec))
-#define WSEC_WEP_ENABLED(wsec)	((wsec) & WEP_ENABLED)
-#define WSEC_TKIP_ENABLED(wsec)	((wsec) & TKIP_ENABLED)
-#define WSEC_AES_ENABLED(wsec)	((wsec) & AES_ENABLED)
-#define WSEC_ENABLED(wsec)	((wsec) & (WEP_ENABLED | TKIP_ENABLED | AES_ENABLED))
+/* WPA authentication mode bitvec */
+#define WPA_AUTH_DISABLED	0x0000	/* Legacy (i.e., non-WPA) */
+#define WPA_AUTH_NONE		0x0001	/* none (IBSS) */
+#define WPA_AUTH_UNSPECIFIED	0x0002	/* over 802.1x */
+#define WPA_AUTH_PSK		0x0004	/* Pre-shared key */
+/*#define WPA_AUTH_8021X 0x0020*/	/* 802.1x, reserved */
+#define WPA2_AUTH_UNSPECIFIED	0x0040	/* over 802.1x */
+#define WPA2_AUTH_PSK		0x0080	/* Pre-shared key */
 
 typedef struct wl_led_info {
 	uint32		index;		/* led index */
@@ -310,7 +281,7 @@ typedef struct wl_wpa_header {
 	char ifname[WL_WPA_MSG_IFNAME_MAX];
 	/* version specific data */
 	/* uint8 data[1]; */
-} wl_wpa_header_t PACKED;
+} wl_wpa_header_t;
 
 #define WL_WPA_HEADER_LEN	(ETHER_HDR_LEN + DOT11_LLC_SNAP_HDR_LEN + 2 + WL_WPA_MSG_IFNAME_MAX)
 
@@ -335,7 +306,7 @@ typedef struct wl_eapol_header {
 	char ifname[WL_WPA_MSG_IFNAME_MAX];
 	/* version specific data */
 	/* uint8 802_1x_msg[1]; */
-} wl_eapol_header_t PACKED;
+} wl_eapol_header_t;
 
 #define WL_EAPOL_HEADER_LEN	(ETHER_HDR_LEN + DOT11_LLC_SNAP_HDR_LEN + 2 + WL_WPA_MSG_IFNAME_MAX)
 
@@ -345,6 +316,20 @@ typedef struct wl_eapol_header {
 /* 802.1x driver message current version */
 #define WL_EAPOL_MSG_VERSION	1
 
+#define WL_SECPVT_DATA_LEN	(ETHER_HDR_LEN + 4 + WL_WPA_MSG_IFNAME_MAX)
+
+/* message header for the private data exchange between nas and wl*/
+typedef struct wl_secpvt_data {
+	struct ether_header eth; /* use the Type field in the eth header with the private type*/
+	uint8 version;
+	uint8 sub_type;
+	uint16 data_len;
+	char ifname[WL_WPA_MSG_IFNAME_MAX];
+	/* version specific data */
+	/* uint8 802_1x_msg[1]; */
+}wl_secpvt_data_t;
+
+
 /* srom read/write struct passed through ioctl */
 typedef struct {
 	uint   byteoff;		/* byte offset */
@@ -387,15 +372,17 @@ typedef struct {
 
 typedef struct {
 	uint16		ver;	/* version of this struct */
+	uint16		len;		/* length in bytes of this structure */
 	uint16		cap;	/* sta's advertized capabilities */
 	uint32 		flags;	/* flags defined below */
 	uint32		idle;	/* time since data pkt rx'd from sta */
 	struct ether_addr	ea;	/* Station address */
 	wl_rateset_t	rateset;	/* rateset in use */
+	uint32		in;		/* seconds elapsed since associated */
+	uint32		listen_interval_inms; /* Min Listen interval in ms for this STA*/
 } sta_info_t;
 
-#define WL_STA_INFO_LEN	300
-#define WL_STA_VER	1
+#define WL_STA_VER	2
 
 /* flags fields */
 #define WL_STA_BRCM	0x01
@@ -404,6 +391,8 @@ typedef struct {
 #define WL_STA_AUTHE	0x08 
 #define WL_STA_ASSOC	0x10 
 #define WL_STA_AUTHO	0x20 
+#define WL_STA_WDS	0x40
+#define WL_WDS_LINKUP	0x80
 
 /* Event messages */
 #define WLC_E_SET_SSID		1
@@ -426,7 +415,8 @@ typedef struct {
 #define WLC_E_MIC_ERROR		18	/* TKIP MIC error occurred */
 #define WLC_E_NDIS_LINK		19	/* NDIS style link indication */
 #define WLC_E_ROAM		20
-#define WLC_E_LAST		21
+#define WLC_E_TXFAIL		21	/* dot11FailedCount (txfail) */
+#define WLC_E_LAST		22
 
 /* Event status codes */
 #define WLC_E_STATUS_SUCCESS		0
@@ -519,8 +509,8 @@ typedef struct wlc_rev_info {
 /* bump this number if you change the ioctl interface */
 #define WLC_IOCTL_VERSION	1
 
-/* maximum length buffer required */
-#define WLC_IOCTL_MAXLEN	8192
+#define	WLC_IOCTL_MAXLEN	8192		/* max length ioctl buffer required */
+#define	WLC_IOCTL_SMLEN		256		/* "small" length ioctl buffer required */
 
 /* common ioctl definitions */
 #define WLC_GET_MAGIC				0
@@ -598,7 +588,7 @@ typedef struct wlc_rev_info {
 #define WLC_GET_RATESET				71
 #define WLC_SET_RATESET				72
 #define WLC_GET_LOCALE				73
-#define WLC_SET_LOCALE				74
+#define WLC_LONGTRAIN				74
 #define WLC_GET_BCNPRD				75
 #define WLC_SET_BCNPRD				76
 #define WLC_GET_DTIMPRD				77
@@ -797,7 +787,8 @@ typedef struct wlc_rev_info {
 #define WLC_SET_AUTH_MODE			270
 #define WLC_GET_WAKEENTRY			271
 #define WLC_SET_WAKEENTRY			272
-#define WLC_LAST				273	/* do not change - use get_var/set_var */
+#define WLC_SET_LOCALE				273
+#define WLC_LAST				274	/* do not change - use get_var/set_var */
 
 /*
  * Minor kludge alert:
@@ -844,6 +835,7 @@ typedef struct wlc_rev_info {
 /* Bit masks for radio disabled status - returned by WL_GET_RADIO */
 #define WL_RADIO_SW_DISABLE	(1<<0)
 #define WL_RADIO_HW_DISABLE	(1<<1)
+#define WL_RADIO_UNASSOC_DISABLE	(1<<2)
 
 /* Override bit for WLC_SET_TXPWR.  if set, ignore other level limits */
 #define WL_TXPWR_OVERRIDE	(1<<31)
@@ -940,6 +932,15 @@ typedef struct tx_power {
 	uint8	txpwr_limit;		/* reg and local power limit for current channel (in .25 dBs)*/
 	uint8	txpwr_local_max;	/* local txpwr regulatory max according to our AP (in .25 dBs)*/
 	uint8   txpwr_local_constraint;	/* local power contraint according to our AP (in .25 dBs)*/
+	uint8	txpwr_chan_reg_max;	/* Regulatory max for this channel */
+	uint8	txpwr_target[2];	/* Latest phy level pwr target for 2.4 and 5 Ghz bands */
+	uint8	txpwr_est_Pout[2];	/* Latest pwr estimate for 2.4 and 5 Ghz bands */
+	uint8	txpwr_opo;		/* On G phy, OFDM power offset */
+	uint8	txpwr_bphy_cck_max;	/* Max CCK power for this band, from SROM */
+	uint8	txpwr_bphy_ofdm_max;	/* Max OFDM power for this band, from SROM */
+	uint8	txpwr_aphy_max;		/* Max power for A band, for current channel, from SROM */
+	int8 	txpwr_antgain[2];	/* Ant gain for each band - from SROM */
+	uint8	txpwr_est_Pout_gofdm;	/* Pwr estimate for 2.4 OFDM */
 } tx_power_t;
 
 /* 802.11h measurement types */
@@ -950,11 +951,17 @@ typedef struct tx_power {
 
 /* 802.11h enforcement levels */
 #define SPECT_MNGMT_OFF         0   /* 11h disabled */
-#define SPECT_MNGMT_LOOSE       1   /* Allow scan lists to contain non-11h AP */
-				    /* when 11h is enabled */
-#define SPECT_MNGMT_STRICT      2   /* Prine out non-11h APs from scan list */
+#define SPECT_MNGMT_LOOSE	1		/* qllow scan lists to contain non-11h AP */
+#define SPECT_MNGMT_STRICT	2		/* prune out non-11h APs from scan list */
 
 
+#define WL_CHAN_VALID_HW	(1 << 0)	/* valid with current HW */
+#define WL_CHAN_VALID_SW	(1 << 1)	/* valid with current country setting */
+#define WL_CHAN_BAND_A		(1 << 2)	/* A-band channel */
+#define WL_CHAN_RADAR		(1 << 3)	/* radar sensitive  channel */
+#define WL_CHAN_INACTIVE	(1 << 4)	/* temporarily out of service due to radar */
+#define WL_CHAN_RADAR_PASSIVE	(1 << 5)	/* radar channel is in passive mode */
+
 /* Message levels */
 #define WL_ERROR_VAL		0x0001
 #define WL_TRACE_VAL		0x0002
@@ -1009,11 +1016,32 @@ typedef struct tx_power {
 #define WL_WDS_WPA_ROLE_SUP	1	/* supplicant */
 #define WL_WDS_WPA_ROLE_AUTO	255	/* auto, based on mac addr value */
 
-/* afterburner_override */
-#define	ABO_AUTO		-1	/* auto - no override */
-#define	ABO_OFF			0	/* force afterburner off */
-#define	ABO_ON			1	/* force afterburner on */
+/* Structures and constants used for "vndr_ie" IOVar interface */
+#define VNDR_IE_CMD_LEN		4	/* length of the set command string: "add", "del" (+ NULL) */
 
-#undef PACKED
+/* 802.11 Mgmt Packet flags */
+#define VNDR_IE_BEACON_FLAG	0x1
+#define VNDR_IE_PRBRSP_FLAG	0x2
+#define VNDR_IE_ASSOCRSP_FLAG	0x4
+#define VNDR_IE_AUTHRSP_FLAG	0x8
+
+typedef struct vndr_ie_info {
+	uint32 pktflag;			/* bitmask indicating which packet(s) contain this IE */
+	vndr_ie_t vndr_ie_data;		/* vendor IE data */
+} vndr_ie_info_t;
+
+typedef struct vndr_ie_buf {
+	int iecount;			/* number of entries in the vndr_ie_list[] array */
+	vndr_ie_info_t vndr_ie_list[1];	/* variable size list of vndr_ie_info_t structs */
+} vndr_ie_buf_t;
+
+typedef struct vndr_ie_setbuf {
+	char cmd[VNDR_IE_CMD_LEN];	/* vndr_ie IOVar set command : "add", "del" + NULL */
+	vndr_ie_buf_t vndr_ie_buffer;	/* buffer containing Vendor IE list information */
+} vndr_ie_setbuf_t;
+
+#if !defined(__GNUC__)
+#pragma pack(pop)
+#endif
 
 #endif /* _wlioctl_h_ */
