@@ -11,6 +11,7 @@
 #include <linux/fs.h>
 #include <linux/proc_fs.h>
 #include <linux/miscdevice.h>
+#include <linux/kmod.h>
 #include <asm/uaccess.h>
 #include <asm/signal.h>
 
@@ -101,6 +102,7 @@ static void gpio_out_pulse_tasklet(unsigned long arg);
 DECLARE_TASKLET(gpioOutPulseTasklet, gpio_out_pulse_tasklet, 0);
 
 static DECLARE_MUTEX(gpio_sem);
+static DECLARE_MUTEX_LOCKED(input_notify_sem);
 
 static void cc_gpio_enableintr(void)
 {
@@ -137,7 +139,7 @@ static void gpio_pin_enableintr(int pinNum, int intrLevel)
 	sb_gpiointmask(gpio_sbh,  (1 << pinNum), (1 << pinNum));
 }
 
-static void gpio_pin_disableintr(pinNum)
+static void gpio_pin_disableintr(int pinNum)
 {
 	/* Disable GPIO interrupt for the pin. */
 	sb_gpiointmask(gpio_sbh, (1 << pinNum), 0);
@@ -161,13 +163,22 @@ gpio_input_tasklet(unsigned long arg)
 		/* Reschedule self */
 		tasklet_schedule(&gpioInputTasklet);
 	} else if (time_after(jiffies, debounceTime)) {
-		latchedInput |= sb_gpioin(gpio_sbh);
+		u32 new_latched_input;
+		u32 old_latched_input;
+
+		new_latched_input = latchedInput | sb_gpioin(gpio_sbh);
+		old_latched_input = latchedInput;
+		latchedInput = new_latched_input;
 
 		/* Since we just read the inputs, reset the debounce timer. */
 		debounceTime = 0;
 
 		/* ISR handled - enable interrupts */
 		cc_gpio_enableintr();
+
+		if (new_latched_input != old_latched_input) {
+			up(&input_notify_sem);
+		}
 	} else {
 		/* Debounce timer not yet elapsed, reschedule self */
 		tasklet_schedule(&gpioInputTasklet);
@@ -228,6 +239,38 @@ static int gpio_out_pulse(void *unused)
 	}
 }
 
+static int button_notify_userland_thread(void *unused)
+{
+	daemonize();
+	strcpy(current->comm, "button_notify_userland");
+	sigfillset(&current->blocked);
+
+	for (;;) {
+		char *argv[4] =
+		{
+			"/sbin/miscio_input_change",
+			NULL
+		};
+		char *envp[3] =
+		{
+			"HOME=/",
+			"PATH=/sbin:/bin:/usr/sbin:/usr/bin",
+			NULL
+		};
+		int value;
+
+		if (down_interruptible(&input_notify_sem))
+			return -ERESTARTSYS;
+		printk(KERN_INFO "GPIO input handling: %s\n", argv[0]);
+		value = call_usermodehelper(argv[0], argv, envp);
+		if (value != 0) {
+			printk(KERN_WARNING
+				   "GPIO input handling user-level notification returned "
+				   "0x%x\n", value);
+		}
+	}
+}
+
 static int
 gpio_open(struct inode *inode, struct file * file)
 {
@@ -567,6 +610,8 @@ static int gpio_config_proc_write(struct file *file, const char *buf,
 		/* Enable interrupt for this GPIO pin for specified level if required. */
 		gpio_pin_enableintr(gpio_proc_config[index].pinNum, 
 						gpio_proc_config[index].intr_level);
+		latchedInput &= ~(1 << gpio_proc_config[index].pinNum);
+		tasklet_schedule(&gpioInputTasklet);
 	} else {
 		gpio_pin_disableintr(gpio_proc_config[index].pinNum);
 	}
@@ -691,6 +736,9 @@ gpio_init(void)
 	/* Start kernel thread for blinking LEDs */
 	kernel_thread(gpio_out_pulse, NULL, 0);
 
+	/* Start kernel thread for notifying userland programs of button presses */
+	kernel_thread(button_notify_userland_thread, NULL, 0);
+	
 	/* Enable chip common interrupts for inputs */
 	cc_gpio_enableintr();
 
