@@ -78,7 +78,8 @@ struct mtd_info *cfi_cmdset_0002(struct map_info *map, int primary)
 
 		/* Wheee. Bring me the head of someone at AMD. */
 #ifdef AMD_BOOTLOC_BUG
-		if (((major << 8) | minor) < 0x3131) {
+		if ((((major << 8) | minor) < 0x3131) || 
+				(((major << 8) | minor) == 0x3133)) {
 			/* CFI version 1.0 => don't trust bootloc */
 			if (cfi->id & 0x80) {
 				printk(KERN_WARNING "%s: JEDEC Device ID is 0x%02X. Assuming broken CFI table.\n", map->name, cfi->id);
