@@ -140,27 +140,51 @@ static int bcm_host_error_reset = 0;
 _static void dump_pci_status_regs(char *tag)
 {
     struct pci_dev *dev;
-    __u32 err_log, imstate, *ptr;
+    __u32 err_log, err_addr, imstate, *ptr;
     __u32 off, val;
 
     printk("%s\n", tag);
 
     // some backplane status regs
+    ptr = __ioremap(0x18004000+0xea8, 4, _CACHE_UNCACHED);
+    err_addr = readl(ptr);
     ptr = __ioremap(0x18004000+0xeb0, 4, _CACHE_UNCACHED);
     err_log = readl(ptr);
     ptr = __ioremap(0x18004000+0xf90, 4, _CACHE_UNCACHED);
     imstate = readl(ptr);
-    printk("\n*** PCI SBIMErrLog = 0x%08x, PCI SBIMState = 0x%08x\n\n",
-		err_log, imstate);
+    printk("\n*** PCI SBIMErrLogA = 0x%08x, PCI SBIMErrLog = 0x%08x, PCI SBIMState = 0x%08x\n\n",
+		err_addr, err_log, imstate);
     
-    printk("Memory Controller registers\n");
+    printk("PCI core SB registers\n");
     for (off = 0xe00; off <= 0xffc; off += 4) {
-	if ((off & 0x0f) == 0) {
-	    printk("\n0x%08x: ", off);
-	}
-	ptr = __ioremap(0x18008000+off, 4, _CACHE_UNCACHED);
-	val = readl(ptr);
-	printk("0x%08x,", val);
+		if ((off & 0x0f) == 0) {
+		    printk("\n0x%08x: ", off);
+		}
+		ptr = __ioremap(0x18004000+off, 4, _CACHE_UNCACHED);
+		val = readl(ptr);
+		printk("0x%08x,", val);
+    }
+    printk("\n\n");
+    
+    printk("Memory Controller SB registers\n");
+    for (off = 0xe00; off <= 0xffc; off += 4) {
+		if ((off & 0x0f) == 0) {
+		    printk("\n0x%08x: ", off);
+		}
+		ptr = __ioremap(0x18008000+off, 4, _CACHE_UNCACHED);
+		val = readl(ptr);
+		printk("0x%08x,", val);
+    }
+    printk("\n\n");
+    
+    printk("Memory Controller Internal registers\n");
+    for (off = 0; off <= 0x054; off += 4) {
+		if ((off & 0x0f) == 0) {
+		    printk("\n0x%08x: ", off);
+		}
+		ptr = __ioremap(0x18008000+off, 4, _CACHE_UNCACHED);
+		val = readl(ptr);
+		printk("0x%08x,", val);
     }
     printk("\n\n");
     
