@@ -1,3 +1,4 @@
+//depot/nasoc/vlan/nasoc/src/linux/linux/net/ipv4/af_inet.c#1 - branch change 1280 (text)
 /*
  * INET		An implementation of the TCP/IP protocol suite for the LINUX
  *		operating system.  INET is implemented using the  BSD Socket
@@ -1180,6 +1181,17 @@ static int __init inet_init(void)
 	ip_mr_init();
 #endif
 
+#if defined(CONFIG_IPSEC)
+	{
+               extern /* void */ int ipsec_init(void);
+		/*
+		 *  Initialise AF_INET ESP and AH protocol support including 
+		 *  e-routing and SA tables
+		 */
+		ipsec_init();
+	}
+#endif /* CONFIG_IPSEC */
+
 	/*
 	 *	Create all the /proc entries.
 	 */
