@@ -11,10 +11,224 @@
  */
 
 #include <typedefs.h>
+#ifdef BCMDRIVER
 #include <osl.h>
+#include <bcmnvram.h>
+#else
+#include <stdio.h>
+#include <string.h>
+#endif
 #include <bcmutils.h>
 #include <bcmendian.h>
 #include <bcmnvram.h>
+#include <bcmdevs.h>
+
+#ifdef BCMDRIVER
+/* copy a pkt buffer chain into a buffer */
+uint
+pktcopy(void *drv, void *p, uint offset, int len, uchar *buf)
+{
+	uint n, ret = 0;
+
+	if (len < 0)
+		len = 4096;	/* "infinite" */
+
+	/* skip 'offset' bytes */
+	for (; p && offset; p = PKTNEXT(drv, p)) {
+		if (offset < (uint)PKTLEN(drv, p))
+			break;
+		offset -= PKTLEN(drv, p);
+	}
+
+	if (!p)
+		return 0;
+
+	/* copy the data */
+	for (; p && len; p = PKTNEXT(drv, p)) {
+		n = MIN((uint)PKTLEN(drv, p) - offset, (uint)len);
+		bcopy(PKTDATA(drv, p) + offset, buf, n);
+		buf += n;
+		len -= n;
+		ret += n;
+		offset = 0;
+	}
+
+	return ret;
+}
+
+/* return total length of buffer chain */
+uint
+pkttotlen(void *drv, void *p)
+{
+	uint total;
+
+	total = 0;
+	for (; p; p = PKTNEXT(drv, p))
+		total += PKTLEN(drv, p);
+	return (total);
+}
+
+void
+pktq_init(struct pktq *q, uint maxlen, const bool prio_map[])
+{
+	q->head = q->tail = NULL;
+	q->maxlen = maxlen;
+	q->len = 0;
+	if (prio_map) {
+		q->priority = TRUE;
+		bcopy(prio_map, q->prio_map, sizeof(q->prio_map));
+	}
+	else
+		q->priority = FALSE;
+}
+
+/* should always check pktq_full before calling pktenq */
+void
+pktenq(struct pktq *q, void *p, bool lifo)
+{
+	void *next, *prev;
+
+	/* allow 10 pkts slack */
+	ASSERT(q->len < (q->maxlen + 10));
+
+	/* Queueing chains not allowed */
+	ASSERT(PKTLINK(p) == NULL);
+
+	/* Queue is empty */
+	if (q->tail == NULL) {
+		ASSERT(q->head == NULL);
+		q->head = q->tail = p;
+	}
+
+	/* Insert at head or tail */
+	else if (q->priority == FALSE) {
+		/* Insert at head (LIFO) */
+		if (lifo) {
+			PKTSETLINK(p, q->head);
+			q->head = p;
+		}
+		/* Insert at tail (FIFO) */
+		else {
+			ASSERT(PKTLINK(q->tail) == NULL);
+			PKTSETLINK(q->tail, p);
+			PKTSETLINK(p, NULL);
+			q->tail = p;
+		}
+	}
+
+	/* Insert by priority */
+	else {
+		/* legal priorities 0-7 */
+		ASSERT(PKTPRIO(p) <= MAXPRIO);
+
+		ASSERT(q->head);
+		ASSERT(q->tail);
+		/* Shortcut to insertion at tail */
+		if (_pktq_pri(q, PKTPRIO(p)) < _pktq_pri(q, PKTPRIO(q->tail)) ||
+		    (!lifo && _pktq_pri(q, PKTPRIO(p)) <= _pktq_pri(q, PKTPRIO(q->tail)))) {
+			prev = q->tail;
+			next = NULL;
+		}
+		/* Insert at head or in the middle */
+		else {
+			prev = NULL;
+			next = q->head;
+		}
+		/* Walk the queue */
+		for (; next; prev = next, next = PKTLINK(next)) {
+			/* Priority queue invariant */
+			ASSERT(!prev || _pktq_pri(q, PKTPRIO(prev)) >= _pktq_pri(q, PKTPRIO(next)));
+			/* Insert at head of string of packets of same priority (LIFO) */
+			if (lifo) {
+				if (_pktq_pri(q, PKTPRIO(p)) >= _pktq_pri(q, PKTPRIO(next)))
+					break;
+			}
+			/* Insert at tail of string of packets of same priority (FIFO) */
+			else {
+				if (_pktq_pri(q, PKTPRIO(p)) > _pktq_pri(q, PKTPRIO(next)))
+					break;
+			}
+		}
+		/* Insert at tail */
+		if (next == NULL) {
+			ASSERT(PKTLINK(q->tail) == NULL);
+			PKTSETLINK(q->tail, p);
+			PKTSETLINK(p, NULL);
+			q->tail = p;
+		}
+		/* Insert in the middle */
+		else if (prev) {
+			PKTSETLINK(prev, p);
+			PKTSETLINK(p, next);
+		}
+		/* Insert at head */
+		else {
+			PKTSETLINK(p, q->head);
+			q->head = p;
+		}
+	}
+
+	/* List invariants after insertion */
+	ASSERT(q->head);
+	ASSERT(PKTLINK(q->tail) == NULL);
+
+	q->len++;
+}
+
+/* dequeue packet at head */
+void*
+pktdeq(struct pktq *q)
+{
+	void *p;
+
+	if ((p = q->head)) {
+		ASSERT(q->tail);
+		q->head = PKTLINK(p);
+		PKTSETLINK(p, NULL);
+		q->len--;
+		if (q->head == NULL)
+			q->tail = NULL;
+	}
+	else {
+		ASSERT(q->tail == NULL);
+	}
+
+	return (p);
+}
+
+/* dequeue packet at tail */
+void*
+pktdeqtail(struct pktq *q)
+{
+	void *p;
+	void *next, *prev;
+
+	if (q->head == q->tail) {  /* last packet on queue or queue empty */
+		p = q->head;
+		q->head = q->tail = NULL;
+		q->len = 0;
+		return(p);
+	}
+
+	/* start walk at head */
+	prev = NULL;
+	next = q->head;
+
+	/* Walk the queue to find prev of q->tail */
+	for (; next; prev = next, next = PKTLINK(next)) {
+		if (next == q->tail)
+			break;
+	}
+
+	ASSERT(prev);
+
+	PKTSETLINK(prev, NULL);
+	q->tail = prev;
+	q->len--;
+	p = next;
+
+	return (p);
+}
 
 unsigned char bcm_ctype[] = {
 	_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,_BCM_C,			/* 0-7 */
@@ -113,104 +327,67 @@ bcm_atoi(char *s)
 	return (n);
 }
 
-void
-deadbeef(char *p, uint len)
+/* return pointer to location of substring 'needle' in 'haystack' */
+char*
+bcmstrstr(char *haystack, char *needle)
 {
-	static uchar meat[] = { 0xde, 0xad, 0xbe, 0xef };
+	int len, nlen;
+	int i;
 
-	while (len-- > 0) {
-		*p = meat[((uint)p) & 3];
-		p++;
-	}
+	if ((haystack == NULL) || (needle == NULL))
+		return (haystack);
+
+	nlen = strlen(needle);
+	len = strlen(haystack) - nlen + 1;
+
+	for (i = 0; i < len; i++)
+		if (bcmp(needle, &haystack[i], nlen) == 0)
+			return (&haystack[i]);
+	return (NULL);
 }
 
-/* pretty hex print a contiguous buffer */
-void
-prhex(char *msg, uchar *buf, uint nbytes)
+char*
+bcmstrcat(char *dest, const char *src)
 {
-	char line[256];
-	char* p;
-	uint i;
-
-	if (msg && (msg[0] != '\0'))
-		printf("%s: ", msg);
-
-	p = line;
-	for (i = 0; i < nbytes; i++) {
-		if (i % 16 == 0) {
-			p += sprintf(p, "%04d: ", i);	/* line prefix */
-		}
-		p += sprintf(p, "%02x ", buf[i]);
-		if (i % 16 == 15) {
-			printf("%s\n", line);		/* flush line */
-			p = line;
-		}
-	}
-
-	/* flush last partial line */
-	if (p != line)
-		printf("%s\n", line);
+	strcpy(&dest[strlen(dest)], src);
+	return (dest);
 }
 
-/* pretty hex print a pkt buffer chain */
-void
-prpkt(char *msg, void *drv, void *p0)
+#if defined(CONFIG_USBRNDIS_RETAIL) || defined(NDIS_MINIPORT_DRIVER)
+/* registry routine buffer preparation utility functions:
+ * parameter order is like strncpy, but returns count
+ * of bytes copied. Minimum bytes copied is null char(1)/wchar(2)
+ */
+ulong
+wchar2ascii(
+	char *abuf,
+	ushort *wbuf,
+	ushort wbuflen,
+	ulong abuflen
+)
 {
-	void *p;
+	ulong copyct = 1;
+	ushort i;
 
-	if (msg && (msg[0] != '\0'))
-		printf("%s: ", msg);
-
-	for (p = p0; p; p = PKTNEXT(drv, p))
-		prhex(NULL, PKTDATA(drv, p), PKTLEN(drv, p));
-}
-
-/* copy a pkt buffer chain into a buffer */
-uint
-pktcopy(void *drv, void *p, uint offset, int len, uchar *buf)
-{
-	uint n, ret = 0;
-
-	if (len < 0)
-		len = 4096;	/* "infinite" */
-
-	/* skip 'offset' bytes */
-	for (; p && offset; p = PKTNEXT(drv, p)) {
-		if (offset < (uint)PKTLEN(drv, p))
-			break;
-		offset -= PKTLEN(drv, p);
-	}
-
-	if (!p)
+	if (abuflen == 0)
 		return 0;
 
-	/* copy the data */
-	for (; p && len; p = PKTNEXT(drv, p)) {
-		n = MIN((uint)PKTLEN(drv, p) - offset, (uint)len);
-		bcopy(PKTDATA(drv, p) + offset, buf, n);
-		buf += n;
-		len -= n;
-		ret += n;
-		offset = 0;
+	/* wbuflen is in bytes */
+	wbuflen /= sizeof(ushort);
+
+	for (i = 0; i < wbuflen; ++i) {
+		if (--abuflen == 0)
+			break;
+		*abuf++ = (char) *wbuf++;
+		++copyct;
 	}
+	*abuf = '\0';
 
-	return ret;
+	return copyct;
 }
+#endif
 
-/* return total length of buffer chain */
-uint
-pkttotlen(void *drv, void *p)
-{
-	uint total;
-
-	total = 0;
-	for (; p; p = PKTNEXT(drv, p))
-		total += PKTLEN(drv, p);
-	return (total);
-}
-
-
-uchar*
+char*
 bcm_ether_ntoa(char *ea, char *buf)
 {
 	sprintf(buf,"%02x:%02x:%02x:%02x:%02x:%02x",
@@ -234,216 +411,124 @@ bcm_ether_atoe(char *p, char *ea)
 	return (i == 6);
 }
 
-/* 
- * Advance from the current 1-byte tag/1-byte length/variable-length value 
- * triple, to the next, returning a pointer to the next.
- */
-bcm_tlv_t *
-bcm_next_tlv(bcm_tlv_t *elt, int *buflen)
+void
+bcm_mdelay(uint ms)
 {
+	uint i;
+
+	for (i = 0; i < ms; i++) {
+		OSL_DELAY(1000);
+	}
+}
+
+/*
+ * Search the name=value vars for a specific one and return its value.
+ * Returns NULL if not found.
+ */
+char*
+getvar(char *vars, char *name)
+{
+	char *s;
 	int len;
 
-	/* validate current elt */
-	if (*buflen < 2) {
-		return NULL;
-	}
-	
-	len = elt->len;
+	len = strlen(name);
 
-	/* validate remaining buflen */
-	if (*buflen >= (2 + len + 2)) {
-		elt = (bcm_tlv_t*)(elt->data + len);
-		*buflen -= (2 + len);
-	} else {
-		elt = NULL;
+	/* first look in vars[] */
+	for (s = vars; s && *s; ) {
+		if ((bcmp(s, name, len) == 0) && (s[len] == '='))
+			return (&s[len+1]);
+
+		while (*s++)
+			;
 	}
-	
-	return elt;
+
+	/* then query nvram */
+	return (BCMINIT(nvram_get)(name));
 }
 
-/* 
- * Traverse a string of 1-byte tag/1-byte length/variable-length value 
- * triples, returning a pointer to the substring whose first element 
- * matches tag.  Stop parsing when we see an element whose ID is greater
- * than the target key. 
+/*
+ * Search the vars for a specific one and return its value as
+ * an integer. Returns 0 if not found.
  */
-bcm_tlv_t *
-bcm_parse_ordered_tlvs(void *buf, int buflen, uint key)
+int
+getintvar(char *vars, char *name)
 {
-	bcm_tlv_t *elt;
-	int totlen;
+	char *val;
 
-	elt = (bcm_tlv_t*)buf;
-	totlen = buflen;
+	if ((val = getvar(vars, name)) == NULL)
+		return (0);
 
-	/* find tagged parameter */
-	while (totlen >= 2) {
-		uint id = elt->id;
-		int len = elt->len;
-		
-		/* Punt if we start seeing IDs > than target key */
-		if (id > key)
-			return(NULL);
-
-		/* validate remaining totlen */
-		if ((id == key) && (totlen >= (len + 2)))
-			return (elt);
-
-		elt = (bcm_tlv_t*)((uint8*)elt + (len + 2));
-		totlen -= (len + 2);
-	}
-	return NULL;
+	return (bcm_strtoul(val, NULL, 0));
 }
 
-
-/* 
- * Traverse a string of 1-byte tag/1-byte length/variable-length value 
- * triples, returning a pointer to the substring whose first element 
- * matches tag
+/*
+ * Search the vars for a specific one and return its value as
+ * a positive integer. Returns -1 if not found or not a number.
  */
-bcm_tlv_t *
-bcm_parse_tlvs(void *buf, int buflen, uint key)
+int
+getvartoint(char *vars, char *name)
 {
-	bcm_tlv_t *elt;
-	int totlen;
+	char *val;
+	uint n;
 
-	elt = (bcm_tlv_t*)buf;
-	totlen = buflen;
+	if ((val = getvar(vars, name)) == NULL)
+		return (-1);
 
-	/* find tagged parameter */
-	while (totlen >= 2) {
-		int len = elt->len;
 
-		/* validate remaining totlen */
-		if ((elt->id == key) && (totlen >= (len + 2)))
-			return (elt);
+	n = 0;
 
-		elt = (bcm_tlv_t*)((uint8*)elt + (len + 2));
-		totlen -= (len + 2);
+	while (*val) {
+		if (bcm_isdigit(*val)) {
+			n = (n * 10) + *val++ - '0';
+		} else {
+			return (-1);
+		}
 	}
-	
-	return NULL;
+
+	return (n);
 }
 
+
+/* Return gpio pin number assigned to the named pin */
+/*
+* Variable should be in format:
+*
+*	gpio<N>=pin_name
+*
+* 'def_pin' is returned if there is no such variable found.
+*/
+uint
+getgpiopin(char *vars, char *pin_name, uint def_pin)
+{
+	char name[] = "gpioXXXX";
+	char *val;
+	uint pin;
+
+	/* Go thru all possibilities till a match in pin name */
+	for (pin = 0; pin < GPIO_NUMPINS; pin ++) {
+		sprintf(name, "gpio%d", pin);
+		val = getvar(vars, name);
+		if (val && !strcmp(val, pin_name))
+			return pin;
+	}
+	return def_pin;
+}
+
+#ifdef BCMDBG
+/* pretty hex print a pkt buffer chain */
 void
-pktq_init(struct pktq *q, uint maxlen, bool priority)
-{
-	q->head = q->tail = NULL;
-	q->priority = priority;
-	q->maxlen = maxlen;
-	q->len = 0;
-}
-
-bool
-pktenq(struct pktq *q, void *p, bool lifo)
-{
-	void *next, *prev;
-
-	/* Queue is full */
-	if (q->len >= q->maxlen)
-		return FALSE;
-
-	/* Queueing chains not allowed */
-	ASSERT(PKTLINK(p) == NULL);
-
-	/* Queue is empty */
-	if (q->tail == NULL) {
-		ASSERT(q->head == NULL);
-		q->head = q->tail = p;
-	}
-
-	/* Insert at head or tail */
-	else if (q->priority == FALSE) {
-		/* Insert at head (LIFO) */
-		if (lifo) {
-			PKTSETLINK(p, q->head);
-			q->head = p;
-		}
-		/* Insert at tail (FIFO) */
-		else {
-			ASSERT(PKTLINK(q->tail) == NULL);
-			PKTSETLINK(q->tail, p);
-			PKTSETLINK(p, NULL);
-			q->tail = p;
-		}
-	}
-
-	/* Insert by priority */
-	else {
-		ASSERT(q->head);
-		ASSERT(q->tail);
-		/* Shortcut to insertion at tail */
-		if (PKTPRIO(p) < PKTPRIO(q->tail) ||
-		    (!lifo && PKTPRIO(p) <= PKTPRIO(q->tail))) {
-			prev = q->tail;
-			next = NULL;
-		}
-		/* Insert at head or in the middle */
-		else {
-			prev = NULL;
-			next = q->head;
-		}
-		/* Walk the queue */
-		for (; next; prev = next, next = PKTLINK(next)) {
-			/* Priority queue invariant */
-			ASSERT(!prev || PKTPRIO(prev) >= PKTPRIO(next));
-			/* Insert at head of string of packets of same priority (LIFO) */
-			if (lifo) {
-				if (PKTPRIO(p) >= PKTPRIO(next))
-					break;
-			}
-			/* Insert at tail of string of packets of same priority (FIFO) */
-			else {
-				if (PKTPRIO(p) > PKTPRIO(next))
-					break;
-			}
-		}
-		/* Insert at tail */
-		if (next == NULL) {
-			ASSERT(PKTLINK(q->tail) == NULL);
-			PKTSETLINK(q->tail, p);
-			PKTSETLINK(p, NULL);
-			q->tail = p;
-		}
-		/* Insert in the middle */
-		else if (prev) {
-			PKTSETLINK(prev, p);
-			PKTSETLINK(p, next);
-		}
-		/* Insert at head */
-		else {
-			PKTSETLINK(p, q->head);
-			q->head = p;
-		}
-	}
-
-	/* List invariants after insertion */
-	ASSERT(q->head);
-	ASSERT(PKTLINK(q->tail) == NULL);
-
-	q->len++;
-	return TRUE;
-}
-
-void*
-pktdeq(struct pktq *q)
+prpkt(char *msg, void *drv, void *p0)
 {
 	void *p;
 
-	if ((p = q->head)) {
-		ASSERT(q->tail);
-		q->head = PKTLINK(p);
-		PKTSETLINK(p, NULL);
-		q->len--;
-		if (q->head == NULL)
-			q->tail = NULL;
-	}
-	else {
-		ASSERT(q->tail == NULL);
-	}
+	if (msg && (msg[0] != '\0'))
+		printf("%s: ", msg);
 
-	return (p);
+	for (p = p0; p; p = PKTNEXT(drv, p))
+		prhex(NULL, PKTDATA(drv, p), PKTLEN(drv, p));
 }
+#endif	/* #ifdef BCMDBG */
+#endif	/* #ifdef BCMDRIVER */
 
 /*******************************************************************************
  * crc8
@@ -501,107 +586,11 @@ static uint8 crc8_table[256] = {
     0xF4, 0x03, 0x4D, 0xBA, 0xD1, 0x26, 0x68, 0x9F
 };
 
-/*
- * Search the name=value vars for a specific one and return its value.
- * Returns NULL if not found.
- */
-char*
-getvar(char *vars, char *name)
-{
-	char *s;
-	int len;
-
-	len = strlen(name);
-
-	/* first look in vars[] */
-	for (s = vars; s && *s; ) {
-		if ((bcmp(s, name, len) == 0) && (s[len] == '='))
-			return (&s[len+1]);
-
-		while (*s++)
-			;
-	}
-
-	/* then query nvram */
-	return (nvram_get(name));
-}
-
-/*
- * Search the vars for a specific one and return its value as
- * an integer. Returns 0 if not found.
- */
-int
-getintvar(char *vars, char *name)
-{
-	char *val;
-
-	if ((val = getvar(vars, name)) == NULL)
-		return (0);
-
-	return (bcm_strtoul(val, NULL, 0));
-}
-
-/*
- * Search the vars for a specific one and return its value as
- * a positive integer. Returns -1 if not found or not a number.
- */
-int
-getvartoint(char *vars, char *name)
-{
-	char *val;
-	uint n;
-
-	if ((val = getvar(vars, name)) == NULL)
-		return (-1);
-
-
-	n = 0;
-
-	while (*val) {
-		if (bcm_isdigit(*val)) {
-			n = (n * 10) + *val++ - '0';
-		} else {
-			return (-1);
-		}
-	}
-
-	return (n);
-}
-
-/* return pointer to location of substring 'needle' in 'haystack' */
-char*
-bcmstrstr(char *haystack, char *needle)
-{
-	int len, nlen;
-	int i;
-
-	if ((haystack == NULL) || (needle == NULL))
-		return (haystack);
-
-	nlen = strlen(needle);
-	len = strlen(haystack) - nlen + 1;
-
-	for (i = 0; i < len; i++)
-		if (bcmp(needle, &haystack[i], nlen) == 0)
-			return (&haystack[i]);
-	return (NULL);
-}
-
-void
-bcm_mdelay(uint ms)
-{
-	uint i;
-
-	for (i = 0; i < ms; i++) {
-		OSL_DELAY(1000);
-	}
-}
-
 #define CRC_INNER_LOOP(n, c, x) \
     (c) = ((c) >> 8) ^ crc##n##_table[((c) ^ (x)) & 0xff]
 
 uint8
-crc8(
+hndcrc8(
 	uint8 *pdata,	/* pointer to array of data to process */
 	uint  nbytes,	/* number of input data bytes to process */
 	uint8 crc	/* either CRC8_INIT_VALUE or previous return value */
@@ -672,7 +661,7 @@ static uint16 crc16_table[256] = {
 };
 
 uint16
-crc16(
+hndcrc16(
     uint8 *pdata,  /* pointer to array of data to process */
     uint nbytes, /* number of input data bytes to process */
     uint16 crc     /* either CRC16_INIT_VALUE or previous return value */
@@ -751,7 +740,7 @@ static uint32 crc32_table[256] = {
 };
 
 uint32
-crc32(
+hndcrc32(
     uint8 *pdata,  /* pointer to array of data to process */
     uint   nbytes, /* number of input data bytes to process */
     uint32 crc     /* either CRC32_INIT_VALUE or previous return value */
@@ -826,4 +815,209 @@ void testcrc32(void)
 }
 #endif
 
+#ifdef BCMDBG
+int
+bcm_format_flags(const bcm_bit_desc_t *bd, uint32 flags, char* buf, int len)
+{
+	int i;
+	char* p = buf;
+	char hexstr[16];
+	int slen = 0;
+	uint32 bit;
+	char* name;
+
+	if (len < 2 || !buf)
+		return 0;
+	
+	buf[0] = '\0';
+	len -= 1;
+
+	for (i = 0; flags != 0; i++) {
+		bit = bd[i].bit;
+		name = bd[i].name;
+		if (bit == 0 && flags) {
+			/* print any unnamed bits */
+			sprintf(hexstr, "0x%X", flags);
+			name = hexstr;
+			flags = 0;	/* exit loop */
+		} else if ((flags & bit) == 0)
+			continue;
+		slen += strlen(name);
+		if (len < slen)
+			break;
+		if (p != buf) p += sprintf(p, " "); /* btwn flag space */
+		strcat(p, name);
+		p += strlen(name);
+		flags &= ~bit;
+		len -= slen;
+		slen = 1;	/* account for btwn flag space */
+	}
+
+	/* indicate the str was too short */
+	if (flags != 0) {
+		if (len == 0)
+			p--;	/* overwrite last char */
+		p += sprintf(p, ">");
+	}
+
+	return (int)(p - buf);
+}
+
+void
+deadbeef(char *p, uint len)
+{
+	static uchar meat[] = { 0xde, 0xad, 0xbe, 0xef };
+
+	while (len-- > 0) {
+		*p = meat[((uintptr)p) & 3];
+		p++;
+	}
+}
+
+/* pretty hex print a contiguous buffer */
+void
+prhex(char *msg, uchar *buf, uint nbytes)
+{
+	char line[256];
+	char* p;
+	uint i;
+
+	if (msg && (msg[0] != '\0'))
+		printf("%s: ", msg);
+
+	p = line;
+	for (i = 0; i < nbytes; i++) {
+		if (i % 16 == 0) {
+			p += sprintf(p, "%04d: ", i);	/* line prefix */
+		}
+		p += sprintf(p, "%02x ", buf[i]);
+		if (i % 16 == 15) {
+			printf("%s\n", line);		/* flush line */
+			p = line;
+		}
+	}
+
+	/* flush last partial line */
+	if (p != line)
+		printf("%s\n", line);
+}
+
+#define BUFSIZE_TODUMP_ATONCE 512
+
+/* dump large strings to console */
+void
+printfbig(char *buf)
+{
+	uint len, max;
+	char c;
+
+	len = strlen(buf);
+
+	max = BUFSIZE_TODUMP_ATONCE;
+
+	while ( len > max) {
+		c = buf[max];
+		buf[max] = '\0';
+		printf("%s", buf);
+		buf[max] = c;
+
+		buf += max; 
+		len -= max;
+	}
+	/*print the remaining string */
+	printf("%s\n", buf);
+	return;
+}	
+#endif /* BCMDBG */
+
+/* 
+ * Advance from the current 1-byte tag/1-byte length/variable-length value 
+ * triple, to the next, returning a pointer to the next.
+ */
+bcm_tlv_t *
+bcm_next_tlv(bcm_tlv_t *elt, int *buflen)
+{
+	int len;
+
+	/* validate current elt */
+	if (*buflen < 2) {
+		return NULL;
+	}
+	
+	len = elt->len;
+
+	/* validate remaining buflen */
+	if (*buflen >= (2 + len + 2)) {
+		elt = (bcm_tlv_t*)(elt->data + len);
+		*buflen -= (2 + len);
+	} else {
+		elt = NULL;
+	}
+	
+	return elt;
+}
+
+/* 
+ * Traverse a string of 1-byte tag/1-byte length/variable-length value 
+ * triples, returning a pointer to the substring whose first element 
+ * matches tag
+ */
+bcm_tlv_t *
+bcm_parse_tlvs(void *buf, int buflen, uint key)
+{
+	bcm_tlv_t *elt;
+	int totlen;
+
+	elt = (bcm_tlv_t*)buf;
+	totlen = buflen;
+
+	/* find tagged parameter */
+	while (totlen >= 2) {
+		int len = elt->len;
+
+		/* validate remaining totlen */
+		if ((elt->id == key) && (totlen >= (len + 2)))
+			return (elt);
+
+		elt = (bcm_tlv_t*)((uint8*)elt + (len + 2));
+		totlen -= (len + 2);
+	}
+	
+	return NULL;
+}
+
+/* 
+ * Traverse a string of 1-byte tag/1-byte length/variable-length value 
+ * triples, returning a pointer to the substring whose first element 
+ * matches tag.  Stop parsing when we see an element whose ID is greater
+ * than the target key. 
+ */
+bcm_tlv_t *
+bcm_parse_ordered_tlvs(void *buf, int buflen, uint key)
+{
+	bcm_tlv_t *elt;
+	int totlen;
+
+	elt = (bcm_tlv_t*)buf;
+	totlen = buflen;
+
+	/* find tagged parameter */
+	while (totlen >= 2) {
+		uint id = elt->id;
+		int len = elt->len;
+		
+		/* Punt if we start seeing IDs > than target key */
+		if (id > key)
+			return(NULL);
+
+		/* validate remaining totlen */
+		if ((id == key) && (totlen >= (len + 2)))
+			return (elt);
+
+		elt = (bcm_tlv_t*)((uint8*)elt + (len + 2));
+		totlen -= (len + 2);
+	}
+	return NULL;
+}
+
 
