@@ -26,7 +26,14 @@
 #include <bcmsrom.h>
 
 /* debug/trace */
+#if defined(BCMDBG)
+#define	SB_ERROR(args)	printf args
+#else
 #define	SB_ERROR(args)
+#endif
+
+
+#define CLOCK_BASE_5350		12500000 /* Specific to 5350*/
 
 typedef uint32 (*sb_intrsoff_t)(void *intr_arg);
 typedef void (*sb_intrsrestore_t)(void *intr_arg, uint32 arg);
@@ -39,7 +46,7 @@ typedef struct sb_info {
 	uint	chippkg;		/* chip package option */
 	uint	boardtype;		/* board type */
 	uint	boardvendor;		/* board vendor id */
-	uint	bus;			/* what bus type we are going through */
+	uint	bustype;		/* what bus type we are going through */
 
 	void	*osh;			/* osl os handle */
 	void	*sdh;			/* bcmsdh handle */
@@ -72,21 +79,22 @@ typedef struct sb_info {
 } sb_info_t;
 
 /* local prototypes */
-static void* sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void *sdh, char **vars, int *varsz);
-static void sb_scan(sb_info_t *si);
+static void* BCMINIT(sb_doattach)(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void *sdh, char **vars, int *varsz);
+static void BCMINIT(sb_scan)(sb_info_t *si);
 static uint sb_corereg(void *sbh, uint coreidx, uint regoff, uint mask, uint val);
 static uint _sb_coreidx(void *sbh);
 static uint sb_findcoreidx(void *sbh, uint coreid, uint coreunit);
-static uint sb_pcidev2chip(uint pcidev);
-static uint sb_chip2numcores(uint chip);
+static uint BCMINIT(sb_pcidev2chip)(uint pcidev);
+static uint BCMINIT(sb_chip2numcores)(uint chip);
 
 #define	SB_INFO(sbh)	(sb_info_t*)sbh
 #define	SET_SBREG(sbh, r, mask, val)	W_SBREG((sbh), (r), ((R_SBREG((sbh), (r)) & ~(mask)) | (val)))
 #define	GOODCOREADDR(x)	(((x) >= SB_ENUM_BASE) && ((x) <= SB_ENUM_LIM) && ISALIGNED((x), SB_CORE_SIZE))
-#define	GOODREGS(regs)	(regs && ISALIGNED(regs, SB_CORE_SIZE))
-#define	REGS2SB(va)	(sbconfig_t*) ((uint)(va) + SBCONFIGOFF)
+#define	GOODREGS(regs)	((regs) && ISALIGNED((uintptr)(regs), SB_CORE_SIZE))
+#define	REGS2SB(va)	(sbconfig_t*) ((int8*)(va) + SBCONFIGOFF)
 #define	GOODIDX(idx)	(((uint)idx) < SB_MAXCORES)
 #define	BADIDX		(SB_MAXCORES+1)
+#define	NOREV		(SBIDH_RC_MASK + 1)
 
 #define	R_SBREG(sbh, sbr)	sb_read_sbreg((sbh), (sbr))
 #define	W_SBREG(sbh, sbr, v)	sb_write_sbreg((sbh), (sbr), (v))
@@ -121,7 +129,6 @@ static uint sb_chip2numcores(uint chip);
 
 #define SCC_LOW2FAST_LIMIT	5000	/* turn on fast clock time, in unit of ms */
 
-
 static uint32
 sb_read_sbreg(void *sbh, volatile uint32 *sbr)
 {
@@ -141,7 +148,7 @@ sb_read_sbreg(void *sbh, volatile uint32 *sbr)
 		INTR_OFF(si, intr_val);
 		tmp = 1;
 		OSL_PCMCIA_WRITE_ATTR(si->osh, MEM_SEG, &tmp, 1);
-		(uint32)sbr &= ~(1 << 11);	/* mask out bit 11*/
+		(uintptr)sbr &= ~(1 << 11);	/* mask out bit 11*/
 	}
 
 	val = R_REG(sbr);
@@ -175,20 +182,20 @@ sb_write_sbreg(void *sbh, volatile uint32 *sbr, uint32 v)
 		INTR_OFF(si, intr_val);
 		tmp = 1;
 		OSL_PCMCIA_WRITE_ATTR(si->osh, MEM_SEG, &tmp, 1);
-		(uint32)sbr &= ~(1 << 11);	/* mask out bit 11 */
+		(uintptr)sbr &= ~(1 << 11);	/* mask out bit 11 */
 	}
 
-	if (si->bus == PCMCIA_BUS) {
+	if (BUSTYPE(si->bustype) == PCMCIA_BUS) {
 #ifdef IL_BIGENDIAN
 		dummy = R_REG(sbr);
-		W_REG((volatile uint16 *)((uint32)sbr + 2), (uint16)((v >> 16) & 0xffff));
+		W_REG(((volatile uint16 *)sbr + 1), (uint16)((v >> 16) & 0xffff));
 		dummy = R_REG(sbr);
 		W_REG((volatile uint16 *)sbr, (uint16)(v & 0xffff));
 #else
 		dummy = R_REG(sbr);
 		W_REG((volatile uint16 *)sbr, (uint16)(v & 0xffff));
 		dummy = R_REG(sbr);
-		W_REG((volatile uint16 *)((uint32)sbr + 2), (uint16)((v >> 16) & 0xffff));
+		W_REG(((volatile uint16 *)sbr + 1), (uint16)((v >> 16) & 0xffff));
 #endif
 	} else
 		W_REG(sbr, v);
@@ -210,25 +217,33 @@ sb_write_sbreg(void *sbh, volatile uint32 *sbr, uint32 v)
  * varsz - pointer to int to return the size of the vars
  */
 void*
-sb_attach(uint devid, void *osh, void *regs, uint bustype, void *sdh, char **vars, int *varsz)
+BCMINITFN(sb_attach)(uint devid, void *osh, void *regs, uint bustype, void *sdh, char **vars, int *varsz)
 {
 	sb_info_t *si;
 
 	/* alloc sb_info_t */
-	if ((si = MALLOC(sizeof (sb_info_t))) == NULL) {
-		SB_ERROR(("sb_attach: malloc failed!\n"));
+	if ((si = MALLOC(osh, sizeof (sb_info_t))) == NULL) {
+		SB_ERROR(("sb_attach: malloc failed! malloced %d bytes\n", MALLOCED(osh)));
 		return (NULL);
 	}
 
-	return (sb_doattach(si, devid, osh, regs, bustype, sdh, vars, varsz));
+	if (BCMINIT(sb_doattach)(si, devid, osh, regs, bustype, sdh, vars, varsz) == NULL) {
+		MFREE(osh, si, sizeof (sb_info_t));
+		return (NULL);
 }
+	return si;
+}
+
+/* Using sb_kattach depends on SB_BUS support, either implicit  */
+/* no limiting BCMBUSTYPE value) or explicit (value is SB_BUS). */
+#if !defined(BCMBUSTYPE) || (BCMBUSTYPE == SB_BUS)
 
 /* global kernel resource */
 static sb_info_t ksi;
 
 /* generic kernel variant of sb_attach() */
 void*
-sb_kattach()
+BCMINITFN(sb_kattach)()
 {
 	uint32 *regs;
 	char *unused;
@@ -239,31 +254,35 @@ sb_kattach()
 
 		regs = (uint32 *)REG_MAP(SB_ENUM_BASE, SB_CORE_SIZE);
 		cid = R_REG((uint32 *)regs);
-		if (((cid & (CID_ID_MASK | CID_PKG_MASK)) == 0x00104712) &&
-		    ((cid & CID_REV_MASK) <= 0x00020000)) {
+		if (((cid & CID_ID_MASK) == BCM4712_DEVICE_ID) &&
+		    ((cid & CID_PKG_MASK) != BCM4712LARGE_PKG_ID) &&
+		    ((cid & CID_REV_MASK) <= (3 << CID_REV_SHIFT))) {
 			uint32 *scc, val;
 
-			scc = (uint32 *)((uint32)regs + OFFSETOF(chipcregs_t, slow_clk_ctl));
+			scc = (uint32 *)((uchar*)regs + OFFSETOF(chipcregs_t, slow_clk_ctl));
 			val = R_REG(scc);
 			SB_ERROR(("    initial scc = 0x%x\n", val));
 			val |= SCC_SS_XTAL;
 			W_REG(scc, val);
 		}
 
-		sb_doattach(&ksi, BCM4710_DEVICE_ID, NULL, (void*)regs,
-			SB_BUS, NULL, &unused, &varsz);
+		if (BCMINIT(sb_doattach)(&ksi, BCM4710_DEVICE_ID, NULL, (void*)regs,
+			SB_BUS, NULL, &unused, &varsz) == NULL) {
+			return NULL;
+		}
 	}
 
 	return &ksi;
 }
+#endif
 
 static void*
-sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void *sdh, char **vars, int *varsz)
+BCMINITFN(sb_doattach)(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void *sdh, char **vars, int *varsz)
 {
 	uint origidx;
 	chipcregs_t *cc;
-	sbconfig_t *sb;
 	uint32 w;
+	int res;
 
 	ASSERT(GOODREGS(regs));
 
@@ -283,32 +302,30 @@ sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void
 			bustype = PCI_BUS;
 	}
 
-	si->bus = bustype;
+	si->bustype = bustype;
+	if (si->bustype != BUSTYPE(si->bustype)) {
+		SB_ERROR(("sb_doattach: bus type %d does not match configured bus type %d\n",
+			  si->bustype, BUSTYPE(si->bustype)));
+		return NULL;
+	}
 
-	if (si->bus == PCMCIA_BUS)
-		/* need to set memseg flag for CF card first before any sb registers access, 
-		 * such as the access inside sb_scan.  the card type is detected and memseg
-		 * flag is reassigned later after srom_var_init.  there should be no effect
-		 * for PCMCIA cards even though the memseg flag is set
-		 */
+	/* need to set memseg flag for CF card first before any sb registers access */
+	if (BUSTYPE(si->bustype) == PCMCIA_BUS)
 		si->memseg = TRUE;
 
 	/* kludge to enable the clock on the 4306 which lacks a slowclock */
-	if (si->bus == PCI_BUS)
+	if (BUSTYPE(si->bustype) == PCI_BUS)
 		sb_pwrctl_xtal((void*)si, XTAL|PLL, ON);
 
 	/* initialize current core index value */
 	si->curidx = _sb_coreidx((void*)si);
-	if (si->curidx == BADIDX)
-		goto bad;
-
-	/* get sonics backplane revision */
-	sb = REGS2SB(si->curmap);
-	si->sonicsrev = (R_SBREG((void*)si, &(sb)->sbidlow) & SBIDL_RV_MASK) >> SBIDL_RV_SHIFT;
+	if (si->curidx == BADIDX) {
+		return NULL;
+	}
 
 	/* keep and reuse the initial register mapping */
 	origidx = si->curidx;
-	if (si->bus == SB_BUS)
+	if (BUSTYPE(si->bustype) == SB_BUS)
 		si->regs[origidx] = regs;
 
 	/* is core-0 a chipcommon core? */
@@ -325,24 +342,24 @@ sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void
 		si->chippkg = (R_REG(&cc->chipid) & CID_PKG_MASK) >> CID_PKG_SHIFT;
 	} else {
 		/* The only pcmcia chip without a chipcommon core is a 4301 */
-		if (si->bus == PCMCIA_BUS)
+		if (BUSTYPE(si->bustype) == PCMCIA_BUS)
 			devid = BCM4301_DEVICE_ID;
 
 		/* no chip common core -- must convert device id to chip id */
-		if ((si->chip = sb_pcidev2chip(devid)) == 0) {
+		if ((si->chip = BCMINIT(sb_pcidev2chip)(devid)) == 0) {
 			SB_ERROR(("sb_attach: unrecognized device id 0x%04x\n", devid));
-			goto bad;
+			return NULL;
 		}
 	}
 
 	/* get chipcommon rev */
-	si->ccrev = cc? sb_corerev((void*)si) : 0;
+	si->ccrev = cc ? sb_corerev((void*)si) : NOREV;
 
 	/* determine numcores */
-	if ((si->ccrev == 4) || (si->ccrev >= 6))
+	if (cc && ((si->ccrev == 4) || (si->ccrev >= 6)))
 		si->numcores = (R_REG(&cc->chipid) & CID_CC_MASK) >> CID_CC_SHIFT;
 	else
-		si->numcores = sb_chip2numcores(si->chip);
+		si->numcores = BCMINIT(sb_chip2numcores)(si->chip);
 
 	/* return to original core */
 	sb_setcoreidx((void*)si, origidx);
@@ -351,14 +368,12 @@ sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void
 	ASSERT(si->chip);
 
 	/* scan for cores */
-	sb_scan(si);
+	BCMINIT(sb_scan)(si);
 
-	/* initialize the vars after sb_scan so that the core rev. information
-	 * collected by sb_scan is available for the srom_var_init.
-	 */
-	if (srom_var_init(si, si->bus, si->curmap, osh, vars, varsz)) {
-		SB_ERROR(("sb_attach: srom_var_init failed\n"));
-		goto bad;
+	/* srom_var_init() depends on sb_scan() info */
+	if ((res = srom_var_init(si, si->bustype, si->curmap, osh, vars, varsz))) {
+		SB_ERROR(("sb_attach: srom_var_init failed: bad srom\n"));
+		return (NULL);
 	}
 	
 	if (cc == NULL) {
@@ -369,17 +384,17 @@ sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void
 		 * For example, the "A0" silicon of each chip is chip rev 0.
 		 * For PCMCIA we get it from the CIS instead.
 		 */
-		if (si->bus == PCMCIA_BUS) {
+		if (BUSTYPE(si->bustype) == PCMCIA_BUS) {
 			ASSERT(vars);
 			si->chiprev = getintvar(*vars, "chiprev");
-		} else if (si->bus == PCI_BUS) {
+		} else if (BUSTYPE(si->bustype) == PCI_BUS) {
 			w = OSL_PCI_READ_CONFIG(osh, PCI_CFG_REV, sizeof (uint32));
 			si->chiprev = w & 0xff;
 		} else
 			si->chiprev = 0;
 	}
 
-	if (si->bus == PCMCIA_BUS) {
+	if (BUSTYPE(si->bustype) == PCMCIA_BUS) {
 		w = getintvar(*vars, "regwindowsz");
 		si->memseg = (w <= CFTABLE_REGWIN_2K) ? TRUE : FALSE;
 	}
@@ -393,11 +408,11 @@ sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void
 	/* gpio control core is required */
 	if (!GOODIDX(si->gpioidx)) {
 		SB_ERROR(("sb_attach: gpio control core not found\n"));
-		goto bad;
+		return NULL;
 	}
 
 	/* get boardtype and boardrev */
-	switch (si->bus) {
+	switch (BUSTYPE(si->bustype)) {
 	case PCI_BUS:
 		/* do a pci config read to get subsystem id and subvendor id */
 		w = OSL_PCI_READ_CONFIG(osh, PCI_CFG_SVID, sizeof (uint32));
@@ -422,13 +437,15 @@ sb_doattach(sb_info_t *si, uint devid, void *osh, void *regs, uint bustype, void
 		ASSERT(si->boardtype);
 	}
 
+#ifdef BCMDBG
 	/* clear any previous epidiag-induced target abort */
 	sb_taclear((void*)si);
+#endif
 
 	return ((void*)si);
 
 bad:
-	MFREE(si, sizeof (sb_info_t));
+	MFREE(si->osh, si, sizeof (sb_info_t));
 	return (NULL);
 }
 
@@ -464,7 +481,7 @@ _sb_coreidx(void *sbh)
 	si = SB_INFO(sbh);
 	ASSERT(si);
 
-	switch (si->bus) {
+	switch (BUSTYPE(si->bustype)) {
 	case SB_BUS:
 		sb = REGS2SB(si->curmap);
 		sbaddr = sb_base(R_SBREG(sbh, &sb->sbadmatch0));
@@ -475,7 +492,7 @@ _sb_coreidx(void *sbh)
 		break;
 
 	case PCMCIA_BUS: {
-		uint8 tmp;
+		uint8 tmp = 0;
 
 		OSL_PCMCIA_READ_ATTR(si->osh, PCMCIA_ADDR0, &tmp, 1);
 		sbaddr  = (uint)tmp << 12;
@@ -519,6 +536,15 @@ sb_corerev(void *sbh)
 	return (R_SBREG(sbh, &(sb)->sbidhigh) & SBIDH_RC_MASK);
 }
 
+void *
+sb_osh(void *sbh)
+{
+	sb_info_t *si;
+
+	si = SB_INFO(sbh);
+	return si->osh;
+}
+
 #define	SBTML_ALLOW	(SBTML_PE | SBTML_FGC | SBTML_FL_MASK)
 
 /* set/clear sbtmstatelow core-specific flags */
@@ -578,7 +604,7 @@ sb_iscoreup(void *sbh)
 	si = SB_INFO(sbh);
 	sb = REGS2SB(si->curmap);
 
-	return ((R_SBREG(sbh, &(sb)->sbtmstatelow) & (SBTML_RESET | SBTML_REJ_MASK | SBTML_CLK)) == SBTML_CLK);
+	return ((R_SBREG(sbh, &(sb)->sbtmstatelow) & (SBTML_RESET | SBTML_REJ | SBTML_CLK)) == SBTML_CLK);
 }
 
 /*
@@ -606,7 +632,7 @@ sb_corereg(void *sbh, uint coreidx, uint regoff, uint mask, uint val)
 	origidx = sb_coreidx(sbh);
 
 	/* switch core */
-	r = (uint32*) ((uint) sb_setcoreidx(sbh, coreidx) + regoff);
+	r = (uint32*) ((uchar*) sb_setcoreidx(sbh, coreidx) + regoff);
 
 	/* mask and set */
 	if (mask || val) {
@@ -635,7 +661,7 @@ sb_corereg(void *sbh, uint coreidx, uint regoff, uint mask, uint val)
 
 /* scan the sb enumerated space to identify all cores */
 static void
-sb_scan(sb_info_t *si)
+BCMINITFN(sb_scan)(sb_info_t *si)
 {
 	void *sbh;
 	uint origidx;
@@ -649,7 +675,8 @@ sb_scan(sb_info_t *si)
 	/* save current core index */
 	origidx = sb_coreidx(sbh);
 
-	si->pciidx = si->gpioidx = BADIDX;
+	si->pciidx = si->pcmciaidx = si->gpioidx = BADIDX;
+	si->pcirev = si->pcmciarev = NOREV;
 
 	for (i = 0; i < si->numcores; i++) {
 		sb_setcoreidx(sbh, i);
@@ -681,7 +708,8 @@ sb_scan(sb_info_t *si)
 	} else if (sb_findcoreidx(sbh, SB_EXTIF, 0)) {
 		si->gpioidx = sb_findcoreidx(sbh, SB_EXTIF, 0);
 		si->gpioid = SB_EXTIF;
-	}
+	} else
+		ASSERT(si->gpioidx != BADIDX);
 
 	/* return to original core index */
 	sb_setcoreidx(sbh, origidx);
@@ -699,19 +727,19 @@ sb_detach(void *sbh)
 	if (si == NULL)
 		return;
 
-	if (si->bus == SB_BUS)
+	if (BUSTYPE(si->bustype) == SB_BUS)
 		for (idx = 0; idx < SB_MAXCORES; idx++)
 			if (si->regs[idx]) {
 				REG_UNMAP(si->regs[idx]);
 				si->regs[idx] = NULL;
 			}
 
-	MFREE(si, sizeof (sb_info_t));
+	MFREE(si->osh, si, sizeof (sb_info_t));
 }
 
 /* use pci dev id to determine chip id for chips not having a chipcommon core */
 static uint
-sb_pcidev2chip(uint pcidev)
+BCMINITFN(sb_pcidev2chip)(uint pcidev)
 {
 	if ((pcidev >= BCM4710_DEVICE_ID) && (pcidev <= BCM47XX_USB_ID))
 		return (BCM4710_DEVICE_ID);
@@ -719,6 +747,8 @@ sb_pcidev2chip(uint pcidev)
 		return (BCM4610_DEVICE_ID);
 	if ((pcidev >= BCM4402_DEVICE_ID) && (pcidev <= BCM4402_V90_ID))
 		return (BCM4402_DEVICE_ID);
+	if (pcidev == BCM4401_ENET_ID)
+		return (BCM4402_DEVICE_ID);
 	if ((pcidev >= BCM4307_V90_ID) && (pcidev <= BCM4307_D11B_ID))
 		return (BCM4307_DEVICE_ID);
 	if (pcidev == BCM4301_DEVICE_ID)
@@ -729,7 +759,7 @@ sb_pcidev2chip(uint pcidev)
 
 /* convert chip number to number of i/o cores */
 static uint
-sb_chip2numcores(uint chip)
+BCMINITFN(sb_chip2numcores)(uint chip)
 {
 	if (chip == 0x4710)
 		return (9);
@@ -799,7 +829,7 @@ sb_setcoreidx(void *sbh, uint coreidx)
 
 	sbaddr = SB_ENUM_BASE + (coreidx * SB_CORE_SIZE);
 
-	switch (si->bus) {
+	switch (BUSTYPE(si->bustype)) {
 	case SB_BUS:
 		/* map new one */
 		if (!si->regs[coreidx]) {
@@ -851,7 +881,7 @@ sb_setcore(void *sbh, uint coreid, uint coreunit)
 
 /* return chip number */
 uint
-sb_chip(void *sbh)
+BCMINITFN(sb_chip)(void *sbh)
 {
 	sb_info_t *si;
 
@@ -861,7 +891,7 @@ sb_chip(void *sbh)
 
 /* return chip revision number */
 uint
-sb_chiprev(void *sbh)
+BCMINITFN(sb_chiprev)(void *sbh)
 {
 	sb_info_t *si;
 
@@ -871,7 +901,7 @@ sb_chiprev(void *sbh)
 
 /* return chip common revision number */
 uint
-sb_chipcrev(void *sbh)
+BCMINITFN(sb_chipcrev)(void *sbh)
 {
 	sb_info_t *si;
 
@@ -881,7 +911,7 @@ sb_chipcrev(void *sbh)
 
 /* return chip package option */
 uint
-sb_chippkg(void *sbh)
+BCMINITFN(sb_chippkg)(void *sbh)
 {
 	sb_info_t *si;
 
@@ -891,7 +921,7 @@ sb_chippkg(void *sbh)
 
 /* return PCI core rev. */
 uint
-sb_pcirev(void *sbh)
+BCMINITFN(sb_pcirev)(void *sbh)
 {
 	sb_info_t *si;
 
@@ -901,7 +931,7 @@ sb_pcirev(void *sbh)
 
 /* return PCMCIA core rev. */
 uint
-sb_pcmciarev(void *sbh)
+BCMINITFN(sb_pcmciarev)(void *sbh)
 {
 	sb_info_t *si;
 
@@ -911,7 +941,7 @@ sb_pcmciarev(void *sbh)
 
 /* return board vendor id */
 uint
-sb_boardvendor(void *sbh)
+BCMINITFN(sb_boardvendor)(void *sbh)
 {
 	sb_info_t *si;
 
@@ -921,14 +951,14 @@ sb_boardvendor(void *sbh)
 
 /* return boardtype */
 uint
-sb_boardtype(void *sbh)
+BCMINITFN(sb_boardtype)(void *sbh)
 {
 	sb_info_t *si;
 	char *var;
 
 	si = SB_INFO(sbh);
 
-	if (si->bus == SB_BUS && si->boardtype == 0xffff) {
+	if (BUSTYPE(si->bustype) == SB_BUS && si->boardtype == 0xffff) {
 		/* boardtype format is a hex string */
 		si->boardtype = getintvar(NULL, "boardtype");
 
@@ -962,17 +992,17 @@ sb_boardtype(void *sbh)
 
 /* return board bus style */
 uint
-sb_boardstyle(void *sbh)
+BCMINITFN(sb_boardstyle)(void *sbh)
 {
 	sb_info_t *si;
 	uint16 w;
 
 	si = SB_INFO(sbh);
 
-	if (si->bus == PCMCIA_BUS)
+	if (si->bustype == PCMCIA_BUS)
 		return (BOARDSTYLE_PCMCIA);
 
-	if (si->bus == SB_BUS)
+	if (si->bustype == SB_BUS)
 		return (BOARDSTYLE_SOC);
 
 	/* bus is PCI */
@@ -980,7 +1010,7 @@ sb_boardstyle(void *sbh)
 	if (OSL_PCI_READ_CONFIG(si->osh, PCI_CFG_CIS, sizeof (uint32)) != 0)
 		return (BOARDSTYLE_CARDBUS);
 
-	if ((srom_read(si->bus, si->curmap, si->osh, (SPROM_SIZE - 1) * 2, 2, &w) == 0) &&
+	if ((srom_read(si->bustype, si->curmap, si->osh, (SPROM_SIZE - 1) * 2, 2, &w) == 0) &&
 		(w == 0x0313))
 		return (BOARDSTYLE_CARDBUS);
 
@@ -994,7 +1024,7 @@ sb_bus(void *sbh)
 	sb_info_t *si;
 
 	si = SB_INFO(sbh);
-	return (si->bus);
+	return (si->bustype);
 }
 
 /* return list of found cores */
@@ -1021,6 +1051,7 @@ sb_coreregs(void *sbh)
 	return (si->curmap);
 }
 
+#if defined(BCMDBG)
 /* traverse all cores to find and clear source of serror */
 static void
 sb_serr_clear(void *sbh)
@@ -1055,8 +1086,10 @@ sb_serr_clear(void *sbh)
 	INTR_RESTORE(si, intr_val);
 }
 
-/* check if any inband, outband or timeout errors has happened and clear them */
-/* !! must be called with chip clk on */
+/*
+ * Check if any inband, outband or timeout errors has happened and clear them.
+ * Must be called with chip clk on !
+ */
 bool
 sb_taclear(void *sbh)
 {
@@ -1071,7 +1104,7 @@ sb_taclear(void *sbh)
 
 	si = SB_INFO(sbh);	
 
-	if (si->bus == PCI_BUS) {
+	if (BUSTYPE(si->bustype) == PCI_BUS) {
 		volatile uint32 stcmd;
 
 		/* inband error is Target abort for PCI */
@@ -1097,7 +1130,7 @@ sb_taclear(void *sbh)
 			timeout = imstate & SBIM_TO;
 		}
 
-	} else if (si->bus == PCMCIA_BUS) {
+	} else if (BUSTYPE(si->bustype) == PCMCIA_BUS) {
 
 		INTR_OFF(si, intr_val);
 		origidx = sb_coreidx(sbh);
@@ -1126,7 +1159,7 @@ sb_taclear(void *sbh)
 		sb_setcoreidx(sbh, origidx);
 		INTR_RESTORE(si, intr_val);
 
-	} else if (si->bus == SDIO_BUS) {
+	} else if (BUSTYPE(si->bustype) == SDIO_BUS) {
 
 		INTR_OFF(si, intr_val);
 		origidx = sb_coreidx(sbh);
@@ -1154,20 +1187,20 @@ sb_taclear(void *sbh)
 		INTR_RESTORE(si, intr_val);
 	}
 	
-	if ((inband | timeout | serror) != 0) {
+	if (inband | timeout | serror) {
 		rc = TRUE;
 		SB_ERROR(("sb_taclear: inband 0x%x, serror 0x%x, timeout 0x%x!\n", inband, serror, timeout));
 	}
 
 	return (rc);
 }
+#endif	/* BCMDBG */
 
 /* do buffered registers update */
 void
 sb_commit(void *sbh)
 {
 	sb_info_t *si;
-	sbpciregs_t *pciregs;
 	uint origidx;
 	uint intr_val = 0;
 
@@ -1177,12 +1210,23 @@ sb_commit(void *sbh)
 	ASSERT(GOODIDX(origidx));
 
 	INTR_OFF(si, intr_val);
-	/* switch over to pci core */
-	pciregs = (sbpciregs_t*) sb_setcore(sbh, SB_PCI, 0);
+
+	/* switch over to chipcommon core if there is one, else use pci */
+	if (si->ccrev != NOREV) {
+		chipcregs_t *ccregs = (chipcregs_t *)sb_setcore(sbh, SB_CC, 0);
+
+		/* do the buffer registers update */
+		W_REG(&ccregs->broadcastaddress, SB_COMMIT);
+		W_REG(&ccregs->broadcastdata, 0x0);
+	} else if (si->pciidx != BADIDX) {
+		sbpciregs_t *pciregs = (sbpciregs_t *)sb_setcore(sbh, SB_PCI, 0);
 
 	/* do the buffer registers update */
 	W_REG(&pciregs->bcastaddr, SB_COMMIT);
 	W_REG(&pciregs->bcastdata, 0x0);
+	} else {
+		ASSERT((si->ccrev != NOREV) && (si->pciidx != BADIDX));
+	}
 
 	/* restore core index */
 	sb_setcoreidx(sbh, origidx);
@@ -1267,13 +1311,13 @@ sb_core_tofixup(void *sbh)
 
 	si = SB_INFO(sbh);
 
-	if (si->pcirev >= 5)
+	if ((si->pciidx == BADIDX) || (si->pcirev >= 5))
 		return;
 
 	ASSERT(GOODREGS(si->curmap));
 	sb = REGS2SB(si->curmap);
 
-	if (si->bus == SB_BUS) {
+	if (BUSTYPE(si->bustype) == SB_BUS) {
 		if (sb_coreid(sbh) == SB_PCI) {
 			SET_SBREG(sbh, &sb->sbimconfiglow,
 				  SBIMCL_RTO_MASK | SBIMCL_STO_MASK,
@@ -1324,7 +1368,7 @@ sb_core_disable(void *sbh, uint32 bits)
 		 goto disable;
 
 	/* set target reject and spin until busy is clear (preserve core-specific bits) */
-	OR_SBREG(sbh, &sb->sbtmstatelow, rej); 								   
+	OR_SBREG(sbh, &sb->sbtmstatelow, (rej));
 	dummy = R_SBREG(sbh, &sb->sbtmstatelow);								   
 	OSL_DELAY(1);															   
 	SPINWAIT((R_SBREG(sbh, &sb->sbtmstatehigh) & SBTMH_BUSY), 100000); 	   
@@ -1342,7 +1386,7 @@ sb_core_disable(void *sbh, uint32 bits)
 	}																		   
 																		   
 	/* set reset and reject while enabling the clocks */					   
-	W_SBREG(sbh, &sb->sbtmstatelow, (bits | SBTML_FGC | SBTML_CLK | rej | SBTML_RESET));	 
+	W_SBREG(sbh, &sb->sbtmstatelow, (bits | SBTML_FGC | SBTML_CLK | rej | SBTML_RESET));
 	dummy = R_SBREG(sbh, &sb->sbtmstatelow);								   
 	OSL_DELAY(10); 														   
 																		   
@@ -1352,7 +1396,7 @@ sb_core_disable(void *sbh, uint32 bits)
 																		   
 	disable:																		   
 	/* leave reset and reject asserted */									   
-	W_SBREG(sbh, &sb->sbtmstatelow, (bits | rej | SBTML_RESET));			   
+	W_SBREG(sbh, &sb->sbtmstatelow, (bits | rej | SBTML_RESET));
 	OSL_DELAY(1);															   
 }																				   
 
@@ -1417,10 +1461,10 @@ sb_pci_setup(void *sbh, uint32 *dmaoffset, uint coremask)
 		*dmaoffset = 0;
 
 	/* if not pci bus, we're done */
-	if (si->bus != PCI_BUS)
+	if (BUSTYPE(si->bustype) != PCI_BUS)
 		return;
 
-	ASSERT(si->pciidx);
+	ASSERT(si->pciidx != BADIDX);
 
 	/* get current core index */
 	idx = si->curidx;
@@ -1448,9 +1492,13 @@ sb_pci_setup(void *sbh, uint32 *dmaoffset, uint coremask)
 		OSL_PCI_WRITE_CONFIG(si->osh, PCI_INT_MASK, sizeof(uint32), w);
 	}
 
-	/* enable prefetch and bursts for sonics-to-pci translation 2 */
+	/* enable prefetch and bursts for dma big window */
 	OR_REG(&pciregs->sbtopci2, (SBTOPCI_PREF|SBTOPCI_BURST));
 
+	/* enable read multiple for dma big window */
+	if (si->pcirev >= 11)
+		OR_REG(&pciregs->sbtopci2, SBTOPCI_RC_READMULTI);
+
 	if (si->pcirev < 5) {
 		SET_SBREG(sbh, &sb->sbimconfiglow, SBIMCL_RTO_MASK | SBIMCL_STO_MASK,
 			(0x3 << SBIMCL_RTO_SHIFT) | 0x2);
@@ -1562,7 +1610,12 @@ sb_clock_rate(uint32 pll_type, uint32 n, uint32 m)
 	n1 = n & CN_N1_MASK;
 	n2 = (n & CN_N2_MASK) >> CN_N2_SHIFT;
 
-	if ((pll_type == PLL_TYPE1) || (pll_type == PLL_TYPE4)) {
+	if (pll_type == PLL_TYPE6) {
+		if (m & CC_T6_MMASK)
+			return CC_T6_M1;
+		else
+			return CC_T6_M0;
+	} else if ((pll_type == PLL_TYPE1) || (pll_type == PLL_TYPE4)|| (pll_type == PLL_TYPE3)) {
 		n1 = factor6(n1);
 		n2 += CC_F5_BIAS;
 	} else if (pll_type == PLL_TYPE2) {
@@ -1570,11 +1623,15 @@ sb_clock_rate(uint32 pll_type, uint32 n, uint32 m)
 		n2 += CC_T2_BIAS;
 		ASSERT((n1 >= 2) && (n1 <= 7));
 		ASSERT((n2 >= 5) && (n2 <= 23));
-	} else if (pll_type == PLL_TYPE3) {
+	} else if (pll_type == PLL_TYPE5) {
 		return (100000000);
 	} else
 		ASSERT((pll_type >= PLL_TYPE1) && (pll_type <= PLL_TYPE4));
-
+	/* Special case for 5350 */
+	if (pll_type == PLL_TYPE3) { 
+		clock =  CLOCK_BASE_5350 * n1 * n2;
+	}
+	else 
 	clock = CC_CLOCK_BASE * n1 * n2;
 
 	if (clock == 0)
@@ -1585,9 +1642,9 @@ sb_clock_rate(uint32 pll_type, uint32 n, uint32 m)
 	m3 = (m & CC_M3_MASK) >> CC_M3_SHIFT;
 	mc = (m & CC_MC_MASK) >> CC_MC_SHIFT;
 
-	if ((pll_type == PLL_TYPE1) || (pll_type == PLL_TYPE4)) {
+	if ((pll_type == PLL_TYPE1) || (pll_type == PLL_TYPE4) || (pll_type == PLL_TYPE3)) {
 		m1 = factor6(m1);
-		if (pll_type == PLL_TYPE1)
+		if ((pll_type == PLL_TYPE1) || (pll_type == PLL_TYPE3))
 			m2 += CC_F5_BIAS;
 		else
 			m2 = factor6(m2);
@@ -1647,6 +1704,11 @@ sb_clock(void *sbh)
 	} else if ((cc = (chipcregs_t *) sb_setcore(sbh, SB_CC, 0))) {
 		pll_type = R_REG(&cc->capabilities) & CAP_PLL_MASK;
 		n = R_REG(&cc->clockcontrol_n);
+		if (pll_type == PLL_TYPE6)
+			m = R_REG(&cc->clockcontrol_mips);
+		else if (pll_type == PLL_TYPE3)
+			m = R_REG(&cc->clockcontrol_m2);
+		else
 		m = R_REG(&cc->clockcontrol_sb);
 	} else {
 		INTR_RESTORE(si, intr_val);
@@ -1656,6 +1718,9 @@ sb_clock(void *sbh)
 	/* calculate rate */
 	rate = sb_clock_rate(pll_type, n, m);
 
+	if (pll_type == PLL_TYPE3)
+		rate = rate / 2;
+
 	/* switch back to previous core */
 	sb_setcoreidx(sbh, idx);
 
@@ -1838,7 +1903,6 @@ sb_gpiointmask(void *sbh, uint32 mask, uint32 val)
 	return (sb_corereg(sbh, si->gpioidx, regoff, mask, val));
 }
 
-
 #ifdef BCMDBG
 void
 sb_dump(void *sbh, char *buf)
@@ -1854,6 +1918,8 @@ sb_dump(void *sbh, char *buf)
 		si->osh, si->curmap);
 	buf += sprintf(buf, "sonicsrev %d ccrev %d pcirev %d gpioidx %d gpioid 0x%x\n",
 		si->sonicsrev, si->ccrev, si->pcirev, si->gpioidx, si->gpioid);
+	buf += sprintf(buf, "ccrev %d pcirev %d gpioidx %d gpioid 0x%x\n",
+		si->ccrev, si->pcirev, si->gpioidx, si->gpioid);
 	buf += sprintf(buf, "cores:  ");
 	for (i = 0; i < si->numcores; i++)
 		buf += sprintf(buf, "0x%x ", si->coreid[i]);
@@ -1876,7 +1942,6 @@ sb_dumpregs(void *sbh, char *buf)
 }
 #endif	/* BCMDBG */
 
-
 /*
  * Return the slow clock source.
  * Three sources of SLOW CLOCK: LPO, Xtal, PCI
@@ -1893,7 +1958,7 @@ sb_slowclk_src(void *sbh)
 	ASSERT(sb_coreid(sbh) == SB_CC);
 
 	if (si->ccrev < 6) {
-		switch (si->bus) {
+		switch (BUSTYPE(si->bustype)) {
 			case PCMCIA_BUS: return (SCC_SS_XTAL);
 			case PCI_BUS:
 				v = OSL_PCI_READ_CONFIG(si->osh, PCI_GPIO_OUT, sizeof (uint32));
@@ -1964,10 +2029,12 @@ sb_slowclk_freq(void *sbh, bool max)
 static void
 sb_pwrctl_setdelay(void *sbh, void *chipcregs)
 {
+	sb_info_t *si;
 	chipcregs_t * cc;
 	uint slowmaxfreq, pll_delay, slowclk;
 	uint pll_on_delay, fref_sel_delay;
 
+	si = SB_INFO(sbh);
 	pll_delay = PLL_DELAY;
 
 	/* If the slow clock is not sourced by the xtal then add the xtal_on_delay
@@ -1977,7 +2044,8 @@ sb_pwrctl_setdelay(void *sbh, void *chipcregs)
 	if (slowclk != SCC_SS_XTAL)
 		pll_delay += XTAL_ON_DELAY;
 
-	slowmaxfreq = sb_slowclk_freq(sbh, TRUE);
+	/* Starting with 4318 it is ILP that is used for the delays */
+	slowmaxfreq = sb_slowclk_freq(sbh, (si->ccrev >= 10) ? FALSE : TRUE);
 
 	pll_on_delay = ((slowmaxfreq * pll_delay) + 999999) / 1000000;
 	fref_sel_delay = ((slowmaxfreq * FREF_DELAY) + 999999) / 1000000;
@@ -2042,7 +2110,7 @@ sb_pwrctl_init(void *sbh)
 
 	si = SB_INFO(sbh);
 
-	if (si->bus == SB_BUS)
+	if (si->bustype == SB_BUS)
 		return;
 
 	origidx = si->curidx;
@@ -2054,7 +2122,7 @@ sb_pwrctl_init(void *sbh)
 		goto done;
 
 	/* 4317pc does not work with SlowClock less than 5Mhz */
-	if (si->bus == PCMCIA_BUS) {
+	if (BUSTYPE(si->bustype) == PCMCIA_BUS) {
 		if ((si->ccrev >= 6) && (si->ccrev < 10))
 			SET_REG(&cc->slow_clk_ctl, SCC_CD_MASK, (SCC_DEF_DIV << SCC_CD_SHF));
 	}
@@ -2080,7 +2148,7 @@ sb_pwrctl_fast_pwrup_delay(void *sbh)
 	fpdelay = 0;
 	origidx = si->curidx;
 
-	if (si->bus == SB_BUS)
+	if (BUSTYPE(si->bustype) == SB_BUS)
 		goto done;
 
 	INTR_OFF(si, intr_val);
@@ -2109,7 +2177,7 @@ sb_pwrctl_xtal(void *sbh, uint what, bool on)
 
 	si = SB_INFO(sbh);
 
-	switch (si->bus) {
+	switch (BUSTYPE(si->bustype)) {
 
 
 		case PCMCIA_BUS:
@@ -2123,6 +2191,7 @@ sb_pwrctl_xtal(void *sbh, uint what, bool on)
 			outen = OSL_PCI_READ_CONFIG(si->osh, PCI_GPIO_OUTEN, sizeof (uint32));
 
 			/*
+			 * Avoid glitching the clock if GPRS is already using it.
 			 * We can't actually read the state of the PLLPD so we infer it
 			 * by the value of XTAL_PU which *is* readable via gpioin.
 			 */
@@ -2208,7 +2277,7 @@ sb_pwrctl_clk(void *sbh, uint mode)
 		break;
 
 	case CLK_SLOW:	/* force slow clock */
-		if ((si->bus == SDIO_BUS) || (si->bus == PCMCIA_BUS))
+		if ((BUSTYPE(si->bustype) == SDIO_BUS) || (BUSTYPE(si->bustype) == PCMCIA_BUS))
 			return (-1);
 
 		if (si->ccrev >= 6)
@@ -2254,4 +2323,52 @@ sb_register_intr_callback(void *sbh, void *intrsoff_fn, void *intrsrestore_fn, v
 	si->dev_coreid = si->coreid[si->curidx];
 }
 
+#ifdef BCMDBG
+/* dump power control related registers */
+int
+sb_pwrctl_dump(void *sbh, char *buf)
+{
+	sb_info_t *si;
+	chipcregs_t *cc;
+	uint origidx;
+	char *obuf;
+	uint intr_val = 0;
+
+	si = SB_INFO(sbh);
+	obuf = buf;
+	*buf = '\0';
+
+	INTR_OFF(si, intr_val);
+
+	origidx = si->curidx;
+
+	if ((cc = (chipcregs_t*) sb_setcore(sbh, SB_CC, 0)) == NULL) {
+		INTR_RESTORE(si, intr_val);
+		return (0);
+	}
+
+	if (!(R_REG(&cc->capabilities) & CAP_PWR_CTL))
+		goto done;
+
+	buf += sprintf(buf, "pll_on_delay 0x%x fref_sel_delay 0x%x ",
+		cc->pll_on_delay, cc->fref_sel_delay);
+	if ((si->ccrev >= 6) && (si->ccrev < 10))
+		buf += sprintf(buf, "slow_clk_ctl 0x%x ", cc->slow_clk_ctl);
+	if (si->ccrev >= 10) {
+		buf += sprintf(buf, "system_clk_ctl 0x%x ", cc->system_clk_ctl);
+		buf += sprintf(buf, "clkstatestretch 0x%x ", cc->clkstatestretch);
+	}
+	if (BUSTYPE(si->bustype) == PCI_BUS)
+		buf += sprintf(buf, "gpioout 0x%x gpioouten 0x%x ",
+			OSL_PCI_READ_CONFIG(si->osh, PCI_GPIO_OUT, sizeof (uint32)),
+			OSL_PCI_READ_CONFIG(si->osh, PCI_GPIO_OUTEN, sizeof (uint32)));
+	buf += sprintf(buf, "\n");
+
+done:
+	sb_setcoreidx(sbh, origidx);
+	INTR_RESTORE(si, intr_val);
+	return (int)(buf - obuf);
+}
+
+#endif
 
